import { Auswertung, FieldMapping } from '@admin-client/reporting-shared';
import { AuswertungFormE2EComponent } from '../../../components/auswertung/auswertung-form.e2e.component';
import { AuswertungE2EComponent } from '../../../components/auswertung/auswertung.e2e.component';
import { E2EAuswertungHelper } from '../../../helper/auswertung/auswertung.helper';
import { E2EAuswertungVerifier } from '../../../helper/auswertung/auswertung.verifier';
import { dropCollections } from '../../../support/cypress-helper';
import { exist, notExist } from '../../../support/cypress.util';
import { loginAsDaria } from '../../../support/user-util';

describe('Auswertung hinzufügen', () => {
  const page: AuswertungE2EComponent = new AuswertungE2EComponent();
  const form: AuswertungFormE2EComponent = new AuswertungFormE2EComponent();

  const helper: E2EAuswertungHelper = new E2EAuswertungHelper();
  const verifier: E2EAuswertungVerifier = new E2EAuswertungVerifier();

  const fieldMapping0: FieldMapping = {
    sourcePath: '/path/to/source/a',
    targetPath: '/path/to/target/a',
  };

  const fieldMapping1: FieldMapping = {
    sourcePath: '/path/to/source/b',
    targetPath: '/path/to/target/b',
  };

  const auswertung: Auswertung = {
    name: 'Auswertung',
    formIdentifier: {
      formEngineName: 'formEngineName',
      formId: 'formId',
    },
    mappings: [fieldMapping0, fieldMapping1],
  };

  before(() => {
    loginAsDaria();
  });

  after(() => {
    dropCollections();
  });

  it('should show "Weitere Felder auswerten" button', () => {
    helper.openAuswertungen();

    exist(page.getWeitereFelderAuswertenButton());
  });

  it('should show form after button click', () => {
    page.getWeitereFelderAuswertenButton().click();

    exist(form.getRoot());
  });

  it('should navigate to auswertung on cancel', () => {
    form.getCancelButton().click();

    exist(page.getWeitereFelderAuswertenButton());
  });

  it('should add data field in form', () => {
    page.getWeitereFelderAuswertenButton().click();
    form.getAddFieldButton().click();

    exist(form.getDataFieldInput(1));
  });

  it('should fill out form with two data fields', () => {
    helper.fillFormular(auswertung);

    verifier.verifyForm(auswertung);
  });

  it('should delete data fields', () => {
    form.getDataFieldDeleteButton(0).click();

    notExist(form.getDataFieldInput(1));
  });

  it('should show auswertung in list after save', () => {
    form.getSaveButton().click();

    verifier.verifyAuswertungInList(auswertung);
  });
});
