import { Auswertung, FieldMapping } from '@admin-client/reporting-shared';
import { AuswertungFormE2EComponent } from '../../components/auswertung/auswertung-form.e2e.component';
import { AuswertungE2EComponent, AuswertungListItemE2EComponent } from '../../components/auswertung/auswertung.e2e.component';
import { haveText, haveValue } from '../../support/cypress.util';

export class E2EAuswertungVerifier {
  private component: AuswertungE2EComponent = new AuswertungE2EComponent();
  private formComponent: AuswertungFormE2EComponent = new AuswertungFormE2EComponent();

  public verifyFieldMapping(fieldMapping: FieldMapping, index: number): void {
    haveValue(this.formComponent.getSourceMappingFieldInput(index), fieldMapping.sourcePath);
    haveValue(this.formComponent.getTargetMappingFieldInput(index), fieldMapping.targetPath);
  }

  public verifyForm(Auswertung: Auswertung): void {
    haveValue(this.formComponent.getNameInput(), Auswertung.name);
    haveValue(this.formComponent.getFormEngineInput(), Auswertung.formIdentifier.formEngineName);
    haveValue(this.formComponent.getFormIdInput(), Auswertung.formIdentifier.formId);

    Auswertung.mappings.forEach((fieldMapping, index) => {
      this.verifyFieldMapping(fieldMapping, index);
    });
  }

  public verifyAuswertungInList(Auswertung: Auswertung): void {
    const listItem: AuswertungListItemE2EComponent = this.component.getListItem(Auswertung.name);
    haveText(listItem.getName(), Auswertung.name);
    haveText(listItem.getFormEngineName(), Auswertung.formIdentifier.formEngineName);
    haveText(listItem.getFormId(), Auswertung.formIdentifier.formId);
  }
}
