/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ConfigurationLinkRel } from '@admin-client/configuration-shared';
import { ROUTES } from '@admin-client/shared';
import { UserFormComponent } from '@admin-client/user';
import { ApiRootLinkRel } from '@alfa-client/api-root-shared';
import { Route } from '@angular/router';
import { AuswertungFormPageComponent } from '../pages/auswertung/auswertung-form-page/auswertung-form-page.component';
import { AuswertungListPageComponent } from '../pages/auswertung/auswertung-list-page/auswertung-list-page.component';
import { OrganisationsEinheitPageComponent } from '../pages/organisationseinheit/organisationseinheit-page/organisationseinheit-page.component';
import { PostfachPageComponent } from '../pages/postfach/postfach-page/postfach-page.component';
import { UnavailablePageComponent } from '../pages/unavailable/unavailable-page/unavailable-page.component';
import { UserFormPageComponent } from '../pages/user/user-form-page/user-form-page.component';
import { UserListPageComponent } from '../pages/user/user-list-page/user-list-page.component';
import { apiRootGuard, configurationGuard } from './app.guard';

export interface GuardData {
  linkRelName: string;
}

export const appRoutes: Route[] = [
  {
    path: ROUTES.POSTFACH,
    component: PostfachPageComponent,
    title: 'Admin | Postfach',
    runGuardsAndResolvers: 'always',
    canActivate: [configurationGuard],
    data: <GuardData>{ linkRelName: ConfigurationLinkRel.SETTING },
  },
  {
    path: ROUTES.BENUTZER,
    component: UserListPageComponent,
    title: 'Admin | Benutzer',
    runGuardsAndResolvers: 'always',
    canActivate: [apiRootGuard],
    data: <GuardData>{ linkRelName: ApiRootLinkRel.USERS },
  },
  {
    path: ROUTES.BENUTZER_NEU,
    component: UserFormPageComponent,
    title: 'Admin | Benutzer anlegen',
    runGuardsAndResolvers: 'always',
    canActivate: [apiRootGuard],
    data: <GuardData>{ linkRelName: ApiRootLinkRel.USERS },
  },
  {
    path: ROUTES.BENUTZER_ID,
    component: UserFormComponent,
    title: 'Admin | Benutzer bearbeiten',
    runGuardsAndResolvers: 'always',
    canActivate: [apiRootGuard],
    data: <GuardData>{ linkRelName: ApiRootLinkRel.USERS },
  },
  {
    path: ROUTES.ORGANISATIONSEINHEITEN,
    component: OrganisationsEinheitPageComponent,
    title: 'Admin | Organisationseinheiten',
  },
  {
    path: ROUTES.UNAVAILABLE,
    component: UnavailablePageComponent,
    title: 'Unavailable',
  },
  {
    path: ROUTES.AUSWERTUNGEN,
    component: AuswertungListPageComponent,
    title: 'Admin | Auswertung',
    runGuardsAndResolvers: 'always',
    canActivate: [configurationGuard],
    data: <GuardData>{ linkRelName: ConfigurationLinkRel.AUSWERTUNG },
  },
  {
    path: ROUTES.AUSWERTUNGEN_NEU,
    component: AuswertungFormPageComponent,
    title: 'Admin | Auswertung anlegen',
    runGuardsAndResolvers: 'always',
    canActivate: [configurationGuard],
    data: <GuardData>{ linkRelName: ConfigurationLinkRel.AUSWERTUNG },
  },
  {
    path: ROUTES.AUSWERTUNGEN_ID,
    component: AuswertungFormPageComponent,
    title: 'Admin | Auswertung bearbeiten',
    runGuardsAndResolvers: 'always',
    canActivate: [configurationGuard],
    data: <GuardData>{ linkRelName: ConfigurationLinkRel.AUSWERTUNG },
  },
  {
    path: '**',
    component: UnavailablePageComponent,
    title: 'Unavailable',
  },
];
