/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { faker } from '@faker-js/faker';
import {
  AttachmentContainerE2EComponent,
  AttachmentListE2EComponent,
} from 'apps/alfa-e2e/src/components/attachment/attachment.e2e.component';
import { E2EAttachmentHelper } from 'apps/alfa-e2e/src/helper/attachment/attachment.helper';
import { E2EAttachmentVerifier } from 'apps/alfa-e2e/src/helper/attachment/attachment.verifier';
import { BinaryFileSnackbarMessageE2E } from 'apps/alfa-e2e/src/model/binary-file';
import {
  createPostfachNachrichtAttachedItem,
  createPostfachNachrichtReplyItem,
} from 'apps/alfa-e2e/src/support/postfach-nachricht.util';
import { PostfachMailFormularE2EComponent } from '../../../components/postfach/postfach-mail-formular.e2e.component';
import {
  PostfachMailE2EComponent,
  PostfachMailListItemE2EComponent,
} from '../../../components/postfach/postfach-mail.e2e.component';
import { FixedDialogE2EComponent } from '../../../components/ui/fixed-dialog.e2e.component';
import { SnackBarE2EComponent } from '../../../components/ui/snackbar.e2e.component';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { VorgangSubnavigationE2EComponent } from '../../../components/vorgang/vorgang-subnavigation';
import { ClientAttributeNameE2E, ClientAttributesE2E, VorgangE2E } from '../../../model/vorgang';
import {
  PostfachMailItemE2E,
  PostfachNachrichtSnackbarMessageE2E,
  VorgangAttachedItemClientE2E,
  VorgangAttachedItemE2E,
} from '../../../model/vorgang-attached-item';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { PostfachMailPage } from '../../../page-objects/postfach.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { expectIconWithBadge, expectIconWithoutBadge } from '../../../support/angular.util';
import { readFileFromDownloads } from '../../../support/cypress-helper';
import { beChecked, contains, exist, notBeChecked, notBeVisible, notExist, visible } from '../../../support/cypress.util';
import { TEST_FILE_WITH_CONTENT, TEST_FILE_WITH_CONTENT_4_MB, TEST_FILE_WITHOUT_CONTENT } from '../../../support/data.util';
import { initUsermanagerUsers, loginAsSabine } from '../../../support/user-util';
import { initVorgangAttachedItem } from '../../../support/vorgang-attached-item-util';
import {
  buildVorgang,
  createHasNewPostfachNachrichtClientAttribute,
  createHasPostfachNachrichtClientAttribute,
  createVorgang,
  initVorgaenge,
  objectIds,
} from '../../../support/vorgang-util';

describe('PostfachMail', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();
  const snackbar: SnackBarE2EComponent = mainPage.getSnackBar();

  const vorgangPage: VorgangPage = new VorgangPage();
  const postfachMailContainer: PostfachMailE2EComponent = vorgangPage.getPostfachMailcontainer();
  const subnavigation: VorgangSubnavigationE2EComponent = vorgangPage.getSubnavigation();
  const fixedDialog: FixedDialogE2EComponent = vorgangPage.getFixedDialog();
  const postfachMailFormular: PostfachMailFormularE2EComponent = vorgangPage.getPostfachMailFormular();
  const attachmentContainer: AttachmentContainerE2EComponent = postfachMailFormular.getAttachmentContainer();
  const attachmentList: AttachmentListE2EComponent = attachmentContainer.getList();

  const attachmentHelper: E2EAttachmentHelper = new E2EAttachmentHelper();
  const attachmentVerifier: E2EAttachmentVerifier = new E2EAttachmentVerifier();

  const postfachMailPage: PostfachMailPage = new PostfachMailPage();

  const clientAttributes: ClientAttributesE2E = {
    [VorgangAttachedItemClientE2E.OZGCLOUD_NACHRICHTEN_MANAGER]: {
      [ClientAttributeNameE2E.HAS_NEW_POSTFACH_NACHRICHT]: createHasNewPostfachNachrichtClientAttribute(true),
      [ClientAttributeNameE2E.HAS_POSTFACH_NACHRICHT]: createHasPostfachNachrichtClientAttribute(true),
    },
  };

  const vorgang: VorgangE2E = createVorgang();
  const vorgangWithReply: VorgangE2E = {
    ...buildVorgang(objectIds[0], 'VorgangWithReply'),
    clientAttributes,
  };

  const vorgangWithoutPostfach: VorgangE2E = createVorgangWithoutPostfachId();

  function createVorgangWithoutPostfachId(): VorgangE2E {
    return {
      ...buildVorgang(objectIds[2], 'VorgangWithoutPostfachId'),
      eingangs: [...vorgang.eingangs],
      header: { serviceKonto: null, collaborationLevel: 0 },
    };
  }

  const postfachMailReply: PostfachMailItemE2E = createPostfachNachrichtReplyItem();
  const postfachNachrichtAttachedItem: VorgangAttachedItemE2E = {
    ...createPostfachNachrichtAttachedItem(objectIds[1], vorgangWithReply._id.$oid),
    item: postfachMailReply,
  };

  before(() => {
    initVorgaenge([vorgang, vorgangWithoutPostfach, vorgangWithReply]);
    initVorgangAttachedItem([postfachNachrichtAttachedItem]);
    initUsermanagerUsers();

    loginAsSabine();

    waitForSpinnerToDisappear();
    exist(vorgangList.getRoot());
  });

  describe('mail icon', () => {
    it('should not be visible', () => {
      notExist(vorgangList.getListItem(vorgang.name).getPostfachIcon());
    });
  });

  describe('mail icon with badge', () => {
    it('should be visible and have a badge', () => {
      const postfachStatusIcon: HTMLElement = vorgangList.getListItem(vorgangWithReply.name).getPostfachIconMatIcon();
      expectIconWithBadge(postfachStatusIcon);
    });
  });

  describe('navigate to vorgang detail', () => {
    it('should open vorgang detail', () => {
      waitForSpinnerToDisappear();
      vorgangList.getListItem(vorgang.name).getRoot().click();
      waitForSpinnerToDisappear();

      exist(vorgangPage.getVorgangDetailHeader().getRoot());
    });

    it('should show postfach mail button with text', () => {
      exist(postfachMailContainer.getCreateButtonWithText());
    });

    it('should show postfach mail button without text', () => {
      exist(postfachMailContainer.getCreateButtonWithoutText());
    });

    it('should show postfach mail button in subnavigation', () => {
      exist(subnavigation.getPostfachMailIconButton());
    });

    it('should show postfach mail list', () => {
      exist(postfachMailContainer.getList());
    });
  });

  describe('postfach mail', () => {
    const postfachMailToSend: PostfachMailItemE2E = <any>{
      subject: 'Huhu',
      mailBody: faker.lorem.words(10),
    };

    describe('click on button should open dialog', () => {
      it('should show fixed dialog header and content', () => {
        postfachMailContainer.getCreateButtonWithText().click();
        waitForSpinnerToDisappear();

        exist(fixedDialog.getDialogHeader());
        exist(fixedDialog.getDialogContent());
      });

      it('should show postfach mail formular', () => {
        exist(postfachMailFormular.getEmpfaenger());
        exist(postfachMailFormular.getBetreff());
        exist(postfachMailFormular.getText());
        exist(postfachMailFormular.getSendButton());
        exist(postfachMailFormular.getAttachmentContainer().getUploadInput());
        exist(postfachMailFormular.getReplyOption());
      });

      it('should show filled empfaenger', () => {
        contains(postfachMailFormular.getEmpfaenger(), 'Max Testermann');
      });

      it('should close Dialog if click on "X"', () => {
        fixedDialog.getCloseButton().click();

        notExist(postfachMailFormular.getEmpfaenger());
      });

      it('should open dialog click on icon button', () => {
        subnavigation.getPostfachMailIconButton().click();
        waitForSpinnerToDisappear();

        exist(postfachMailFormular.getEmpfaenger());
      });

      it('should hide dialog content click on minimize', () => {
        fixedDialog.getMinMaxButton().click();

        notBeVisible(fixedDialog.getDialogContent());
        exist(fixedDialog.getCloseButton());
      });

      it('should show dialog content click on maximize', () => {
        fixedDialog.getMinMaxButton().click();

        visible(fixedDialog.getDialogContent());
      });
    });

    describe('formular fields', () => {
      it('should show error on subject if send empty', () => {
        postfachMailFormular.getSendButton().click();
        waitForSpinnerToDisappear();

        exist(postfachMailFormular.getBetreffError());
      });

      it('should show validation error on to long subject', () => {
        postfachMailFormular.getBetreff().clear().type(faker.string.alphanumeric(71));
        postfachMailFormular.getSendButton().click();

        exist(postfachMailFormular.getBetreffError());
      });

      it('should show error on text if send empty', () => {
        postfachMailFormular.getSendButton().click();

        exist(postfachMailFormular.getTextError());
      });

      it('should be fillable', () => {
        postfachMailFormular.getBetreff().clear().type(postfachMailToSend.subject);
        postfachMailFormular.getText().clear().type(postfachMailToSend.mailBody);
      });
    });

    describe('attach files', () => {
      it('should show empty attachment after uploading it', () => {
        attachmentHelper.uploadEmptyAttachment();

        attachmentVerifier.verifyAttachmentInList(TEST_FILE_WITHOUT_CONTENT);
      });

      it('should show attachment with content after uploading it', () => {
        attachmentHelper.uploadAttachment();

        attachmentVerifier.verifyAttachmentInList(TEST_FILE_WITH_CONTENT);
      });

      it('should download empty attachment on click', () => {
        attachmentHelper.downloadAttachment(TEST_FILE_WITHOUT_CONTENT);

        attachmentVerifier.verifyAttachmentDownload(TEST_FILE_WITHOUT_CONTENT);
      });

      it('should download attachment with content on click', () => {
        attachmentHelper.downloadAttachment(TEST_FILE_WITH_CONTENT);

        attachmentVerifier.verifyAttachmentDownload(TEST_FILE_WITH_CONTENT);
      });

      it('should not show empty attachment after deleting it', () => {
        attachmentHelper.deleteAttachment(TEST_FILE_WITHOUT_CONTENT);

        attachmentVerifier.verifyAttachmentNotInList(TEST_FILE_WITHOUT_CONTENT);
      });

      it('should delete and multi upload all', () => {
        attachmentHelper.deleteAttachments([TEST_FILE_WITH_CONTENT]);

        attachmentHelper.uploadAttachments([TEST_FILE_WITH_CONTENT, TEST_FILE_WITHOUT_CONTENT]);

        attachmentVerifier.verifyAttachmentsInList([TEST_FILE_WITH_CONTENT, TEST_FILE_WITHOUT_CONTENT]);
      });
    });

    describe('attach file > 3 MB', () => {
      it('should show failed upload', { defaultCommandTimeout: 90000 }, () => {
        attachmentHelper.uploadAttachment(TEST_FILE_WITH_CONTENT_4_MB);

        exist(attachmentList.getLoadingOrErrorItem(TEST_FILE_WITH_CONTENT_4_MB).getRoot());
        contains(
          attachmentList.getLoadingOrErrorItem(TEST_FILE_WITH_CONTENT_4_MB).getRoot(),
          BinaryFileSnackbarMessageE2E.ATTACHMENT_NOT_ADDED.replace('{size}', '3MB'),
        );
      });
    });

    describe('reply option button', () => {
      it('should change state of reply option', () => {
        beChecked(postfachMailFormular.getReplyOption());

        postfachMailFormular.getReplyOption().click();
        notBeChecked(postfachMailFormular.getReplyOption());
      });
    });

    describe('send postfach mail', () => {
      it('should close dialog', () => {
        postfachMailFormular.getSendButton().click();
        waitForSpinnerToDisappear();

        notExist(postfachMailFormular.getEmpfaenger());
      });

      it('should show snackbar', () => {
        contains(snackbar.getMessage(), PostfachNachrichtSnackbarMessageE2E.SEND_SUCCESSFUL);
        notExist(snackbar.getRevokeButton());
      });

      it('should close snackbar on close button', () => {
        snackbar.getCloseButton().click();

        notExist(snackbar.getMessage());
      });

      it('should show postfach mail in list', () => {
        const postfachMailItem: PostfachMailListItemE2EComponent = postfachMailContainer.getListItem(postfachMailToSend.subject);

        exist(postfachMailItem.getRoot());
        exist(postfachMailItem.getUserProfile().getRoot());
        exist(postfachMailItem.getSentAt());
        contains(postfachMailItem.getSubject(), postfachMailToSend.subject);
        contains(postfachMailItem.getText(), postfachMailToSend.mailBody);
      });
    });

    describe('click on postfach mail item with attachment', () => {
      it('should show postfach item list', () => {
        const postfachMailItem: PostfachMailListItemE2EComponent = postfachMailContainer.getListItem(postfachMailToSend.subject);
        postfachMailItem.getRoot().click();
        waitForSpinnerToDisappear();

        exist(postfachMailPage.getRoot());
      });

      it('should contain mail item with attachment', () => {
        const postfachListItem: PostfachMailListItemE2EComponent = postfachMailPage.getListItem(postfachMailToSend.subject);

        exist(postfachListItem.getAttachmentContainer().getList().getItem(TEST_FILE_WITH_CONTENT).getRoot());
      });

      it('should download attachment after click', () => {
        const postfachListItem: PostfachMailListItemE2EComponent = postfachMailPage.getListItem(postfachMailToSend.subject);

        postfachListItem.getAttachmentContainer().getList().getItem(TEST_FILE_WITH_CONTENT).getDownloadButton().click();
        waitForSpinnerToDisappear();

        exist(readFileFromDownloads(TEST_FILE_WITHOUT_CONTENT));
      });

      it('should not contain failed upload', () => {
        const postfachListItem: PostfachMailListItemE2EComponent = postfachMailPage.getListItem(postfachMailToSend.subject);

        notExist(
          postfachListItem.getAttachmentContainer().getList().getLoadingOrErrorItem(TEST_FILE_WITH_CONTENT_4_MB).getRoot(),
        );
      });
    });

    describe('click on back button', () => {
      it('should navigate to vorgang detail', () => {
        postfachMailPage.getSubnavigation().getBackButton().click();
        waitForSpinnerToDisappear();

        exist(vorgangPage.getVorgangDetailHeader().getRoot());
      });

      it('should navigate back to vorgang list', () => {
        vorgangPage.getSubnavigation().getBackButton().click();
        waitForSpinnerToDisappear();

        exist(vorgangList.getRoot());
      });
    });

    describe('mail icon', () => {
      it('should be visible', () => {
        exist(vorgangList.getListItem(vorgang.name).getPostfachIcon());
      });
    });
  });

  describe('navigate to vorgang detail', () => {
    it('should open vorgang detail', () => {
      waitForSpinnerToDisappear();
      vorgangList.getListItem(vorgangWithoutPostfach.name).getRoot().click();
      waitForSpinnerToDisappear();

      exist(vorgangPage.getVorgangDetailHeader().getRoot());
    });

    it('should not show postfach mail button', () => {
      notExist(postfachMailContainer.getCreateButtonWithText());
    });

    it('should not show postfach mail button in subnavigation', () => {
      notExist(subnavigation.getPostfachMailIconButton());
    });

    it('should show text for no postfach attached', () => {
      contains(postfachMailContainer.getNoPostfachText(), 'Dieser Vorgang ist nicht mit einem Postfach verknüpft.');
    });

    it('should navigate back to list', () => {
      vorgangPage.getSubnavigation().getBackButton().click();
      waitForSpinnerToDisappear();

      exist(vorgangList.getRoot());
    });
  });

  describe('navigate to vorgang detail', () => {
    it('should open vorgang detail', () => {
      vorgangList.getListItem(vorgangWithReply.name).getRoot().click();
      waitForSpinnerToDisappear();

      exist(vorgangPage.getVorgangDetailHeader().getRoot());
    });

    it('should show postfach mail button', () => {
      exist(postfachMailContainer.getCreateButtonWithText());
    });

    it('should show postfach mail button in subnavigation', () => {
      exist(subnavigation.getPostfachMailIconButton());
    });

    it('should show postfach mail list', () => {
      exist(postfachMailContainer.getList());
    });

    it('should show postfach reply mail in list', () => {
      exist(postfachMailContainer.getListItem(postfachMailReply.subject).getRoot());
    });

    it('should show item reply icon', () => {
      exist(postfachMailContainer.getListItem(postfachMailReply.subject).getReplyIcon());
    });

    it('should show date', () => {
      exist(postfachMailContainer.getListItem(postfachMailReply.subject).getCreatedAt());
    });

    it('should show text', () => {
      contains(postfachMailContainer.getListItem(postfachMailReply.subject).getText(), postfachMailReply.mailBody);
    });
  });

  describe('click on postfach mail item', () => {
    let postfachMailReplyItem: PostfachMailListItemE2EComponent;

    beforeEach(() => {
      postfachMailReplyItem = postfachMailPage.getListItem(postfachMailReply.subject);
    });

    describe('postfach page', () => {
      it('should open full view of postfachMailList', () => {
        postfachMailReplyItem.getRoot().click();
        waitForSpinnerToDisappear();

        exist(postfachMailPage.getRoot());
      });

      it('should show subnavigation', () => {
        exist(postfachMailPage.getSubnavigation().getBackButton());
      });

      it('should show Nachrichten heading', () => {
        contains(postfachMailPage.getHeading(), postfachMailPage.getHeadingText());
      });
    });

    describe('reply item', () => {
      it('should show item reply icon', () => {
        exist(postfachMailReplyItem.getReplyIcon());
      });

      it('should show reply subject', () => {
        contains(postfachMailReplyItem.getSubject(), postfachMailReply.subject);
      });

      it('should show date', () => {
        exist(postfachMailReplyItem.getCreatedAt());
      });

      it('should show reply text', () => {
        contains(postfachMailReplyItem.getText(), postfachMailReply.mailBody);
      });
    });

    describe('click on back button', () => {
      it('should navigate to vorgang detail', () => {
        postfachMailPage.getSubnavigation().getBackButton().click();
        waitForSpinnerToDisappear();

        exist(vorgangPage.getVorgangDetailHeader().getRoot());
      });
    });
  });

  describe('mail icon', () => {
    it('should navigate back to list', () => {
      vorgangPage.getSubnavigation().getBackButton().click();
      waitForSpinnerToDisappear();

      exist(vorgangList.getRoot());
    });

    it('should be visible and not have a badge', () => {
      const postfachStatusIcon: HTMLElement = vorgangList.getListItem(vorgangWithReply.name).getPostfachIconMatIcon();
      expectIconWithoutBadge(postfachStatusIcon);
    });
  });
});
