import { PostfachMailPage } from 'apps/alfa-e2e/src/page-objects/postfach.po';
import {
  createPostfachNachrichtAttachedItem,
  createPostfachNachrichtReplyItem,
} from 'apps/alfa-e2e/src/support/postfach-nachricht.util';
import { initVorgangAttachedItem } from 'apps/alfa-e2e/src/support/vorgang-attached-item-util';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { E2EPostfachHelper } from '../../../helper/postfach/postfach.helper';
import { E2EPostfachVerifier } from '../../../helper/postfach/postfach.verifier';
import { E2EVorgangNavigator } from '../../../helper/vorgang/vorgang.navigator';
import { ClientAttributeNameE2E, VorgangE2E } from '../../../model/vorgang';
import { PostfachMailItemE2E, VorgangAttachedItemClientE2E, VorgangAttachedItemE2E } from '../../../model/vorgang-attached-item';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { dropCollections } from '../../../support/cypress-helper';
import { exist } from '../../../support/cypress.util';
import { initUsermanagerUsers, loginAsSabine } from '../../../support/user-util';
import {
  buildVorgang,
  createHasNewPostfachNachrichtClientAttribute,
  createHasPostfachNachrichtClientAttribute,
  createVorgang,
  initVorgaenge,
  objectIds,
} from '../../../support/vorgang-util';

describe('Postfach Nachrichten mark unread', () => {
  const vorgangList: VorgangListE2EComponent = new MainPage().getVorgangList();
  const vorgangNavigator: E2EVorgangNavigator = new E2EVorgangNavigator();

  const postfachMailPage: PostfachMailPage = new PostfachMailPage();
  const postfachVerifier: E2EPostfachVerifier = new E2EPostfachVerifier();
  const postfachHelper: E2EPostfachHelper = new E2EPostfachHelper();

  const vorgang: VorgangE2E = createVorgang();
  const vorgangWithReply: VorgangE2E = {
    ...buildVorgang(objectIds[0], 'VorgangWithReply'),
    clientAttributes: {
      [VorgangAttachedItemClientE2E.OZGCLOUD_NACHRICHTEN_MANAGER]: {
        [ClientAttributeNameE2E.HAS_NEW_POSTFACH_NACHRICHT]: createHasNewPostfachNachrichtClientAttribute(true),
        [ClientAttributeNameE2E.HAS_POSTFACH_NACHRICHT]: createHasPostfachNachrichtClientAttribute(true),
      },
    },
  };

  const postfachMailReply: PostfachMailItemE2E = createPostfachNachrichtReplyItem();
  const postfachNachrichtAttachedItem: VorgangAttachedItemE2E = {
    ...createPostfachNachrichtAttachedItem(objectIds[1], objectIds[0]),
    item: postfachMailReply,
  };

  before(() => {
    initVorgaenge([vorgang, vorgangWithReply]);
    initVorgangAttachedItem([postfachNachrichtAttachedItem]);
    initUsermanagerUsers();

    loginAsSabine();

    waitForSpinnerToDisappear();
    exist(vorgangList.getRoot());
  });

  after(() => {
    dropCollections();
  });

  describe('mail icon', () => {
    it('should not have unread badge', () => {
      vorgangNavigator.openVorgang(vorgang.name);

      postfachVerifier.verifyMailIconWithoutBadgeExists();
    });

    it('should have unread badge', () => {
      vorgangNavigator.openVorgang(vorgangWithReply.name);

      postfachVerifier.verifyMailIconWithBadgeExists();
    });

    it('should not have unread badge after messages were read', () => {
      postfachHelper.navigateToPostfach();
      postfachMailPage.getSubnavigation().getBackButton().click();

      postfachVerifier.verifyMailIconWithoutBadgeExists();
    });
  });

  describe('mark as unread button', () => {
    it('should be displayed as action button', () => {
      postfachHelper.navigateToPostfach();

      exist(postfachMailPage.getSubnavigation().getMailUnreadButton());
    });

    it('should navigate back and mark messages as unread', () => {
      postfachMailPage.getSubnavigation().getMailUnreadButton().click();

      postfachVerifier.verifyMailIconWithBadgeExists();
    });

    it('should be displayed as button', () => {
      postfachHelper.navigateToPostfach();

      exist(postfachMailPage.getMailUnreadButton());
    });

    it('should navigate back and mark messages as unread', () => {
      postfachMailPage.getMailUnreadButton().click();

      postfachVerifier.verifyMailIconWithBadgeExists();
    });
  });
});
