import { SnackBarE2EComponent } from '../components/ui/snackbar.e2e.component';
import { VorgangListE2EComponent } from '../components/vorgang/vorgang-list.e2e.component';
import { MainPage, waitForSpinnerToDisappear } from '../page-objects/main.po';
import { VorgangPage } from '../page-objects/vorgang.po';
import { exist, notExist } from '../support/cypress.util';
import { loginAsSabine } from '../support/user-util';
import { E2EAppVerifier } from './app.verifier';

export class E2EAppHelper {
  private readonly mainPage: MainPage = new MainPage();
  private readonly vorgangPage: VorgangPage = new VorgangPage();
  private readonly vorgangList: VorgangListE2EComponent = new VorgangListE2EComponent();

  private readonly verifier: E2EAppVerifier = new E2EAppVerifier();
  private readonly snackBar: SnackBarE2EComponent = new SnackBarE2EComponent();

  public loginAsSabine(): void {
    loginAsSabine();
    this.waitAfterLogin();
  }

  private waitAfterLogin(): void {
    waitForSpinnerToDisappear();
    exist(this.vorgangList.getRoot());
  }

  public navigateToDomain(): void {
    this.mainPage.getHeader().getLogo().click();
  }

  public navigateRelative(url: string): void {
    cy.location('pathname').then((currentPath: string) => cy.visit(`${currentPath}/${url}`));
  }

  public saveAndCloseSnackbar(message: string): void {
    this.verifier.verifySnackbar(message);
    this.closeSnackbar();
  }

  public saveAndCloseRevokeableSnackbar(message: string): void {
    this.verifier.verifyRevokableSnackbar(message);
    this.closeSnackbar();
  }

  private closeSnackbar(): void {
    this.snackBar.getCloseButton().click();
    notExist(this.snackBar.getMessage());
    waitForSpinnerToDisappear();
  }

  public revokeAndCloseSnackbar(message: string): void {
    this.verifier.verifySnackbar(message);
    this.snackBar.getRevokeButton().click();
    exist(this.vorgangPage.getProgressBar());
    waitForSpinnerToDisappear();
  }
}
