/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { CreateFileDataE2E, FileDataE2E, GridFsChunkE2E, GridFsFileE2E, OzgCloudFileE2E } from '../model/binary-file';
import { initFileData } from './cypress-helper';

const xmlRepresentation: FileDataE2E = require('../fixtures/representation/xml.json');

const jpgAttachment: FileDataE2E = require('../fixtures/attachment/jpg.json');
const pdfAttachment: FileDataE2E = require('../fixtures/attachment/pdf.json');

const xmlFile: GridFsFileE2E = require('../fixtures/grid-fs/xml/file.json');
const xmlChunk: GridFsChunkE2E = require('../fixtures/grid-fs/xml/chunk.json');
const xmlOzgCloudFile: OzgCloudFileE2E = require('../fixtures/ozgcloud-file/xml-file.json');

const jpgFile: GridFsFileE2E = require('../fixtures/grid-fs/jpg/file.json');
const jpgChunk: GridFsChunkE2E = require('../fixtures/grid-fs/jpg/chunk.json');
const jpgOzgCloudFile: OzgCloudFileE2E = require('../fixtures/ozgcloud-file/jpg-file.json');

const pdfFile: GridFsFileE2E = require('../fixtures/grid-fs/pdf/file.json');
const pdfChunk: GridFsChunkE2E = require('../fixtures/grid-fs/pdf/chunk.json');
const pdfOzgCloudFile: OzgCloudFileE2E = require('../fixtures/ozgcloud-file/pdf-file.json');

export function createXmlRepresentation(): FileDataE2E {
  return xmlRepresentation;
}

export function createJpgAttachment(): FileDataE2E {
  return jpgAttachment;
}

export function createPdfAttachment(): FileDataE2E {
  return pdfAttachment;
}

export function createXmlFileData(): CreateFileDataE2E {
  return { file: xmlFile, chunks: [xmlChunk], ozgcloudFile: xmlOzgCloudFile };
}

export function createJpgFileData(): CreateFileDataE2E {
  return { file: jpgFile, chunks: [jpgChunk], ozgcloudFile: jpgOzgCloudFile };
}

export function createPdfFileData(): CreateFileDataE2E {
  return { file: pdfFile, chunks: [pdfChunk], ozgcloudFile: pdfOzgCloudFile };
}

export function createFileData(createFileDataE2E: CreateFileDataE2E, id: string, vorgangId: string): CreateFileDataE2E {
  return {
    file: { ...createFileDataE2E.file, _id: { $oid: id }, metadata: { ...createFileDataE2E.file.metadata, vorgangId } },
    chunks: createFileDataE2E.chunks.map((chunk: GridFsChunkE2E) => {
      return { ...chunk, _id: { $oid: id }, files_id: { $oid: id } };
    }),
    ozgcloudFile: { ...createFileDataE2E.ozgcloudFile, _id: { $oid: id }, vorgangId },
  };
}

export function initFile(createFileData: CreateFileDataE2E[]): void {
  initFileData(createFileData);
}
