#!/bin/bash
#
# Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
# Ministerpräsidenten des Landes Schleswig-Holstein
# Staatskanzlei
# Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
#
# Lizenziert unter der EUPL, Version 1.2 oder - sobald
# diese von der Europäischen Kommission genehmigt wurden -
# Folgeversionen der EUPL ("Lizenz");
# Sie dürfen dieses Werk ausschließlich gemäß
# dieser Lizenz nutzen.
# Eine Kopie der Lizenz finden Sie hier:
#
# https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
#
# Sofern nicht durch anwendbare Rechtsvorschriften
# gefordert oder in schriftlicher Form vereinbart, wird
# die unter der Lizenz verbreitete Software "so wie sie
# ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
# ausdrücklich oder stillschweigend - verbreitet.
# Die sprachspezifischen Genehmigungen und Beschränkungen
# unter der Lizenz sind dem Lizenztext zu entnehmen.
#


SCRIPT_DIR=$(cd $(dirname "${BASH_SOURCE[0]}") && pwd)
DOCKER_COMPOSE_CMD="docker compose -f ${SCRIPT_DIR}/docker-compose.yml"

if nc -z localhost 9091 &> /dev/null
then
	echo "VorgangManager already running. Exiting."
	exit
fi

echo "Using $DOCKER_COMPOSE_CMD ..."
echo

if [[ "$OSTYPE" == "linux-gnu"* ]]
then
	export DOCKER_GATEWAY_HOST=172.17.0.1
fi

$DOCKER_COMPOSE_CMD pull

$DOCKER_COMPOSE_CMD up -d mongodb user-manager elastic smocker --wait

echo
echo "Starting VorgangManager to init search index."

SPRING_PROFILE=local,e2e,initSearchIndex docker compose -f ${SCRIPT_DIR}/docker-compose.yml up -d vorgang-manager
echo

waitForInitSearchIndex() {
	($DOCKER_COMPOSE_CMD logs | grep -F -q 'Successful filled up index - exiting')
}

echo "Waiting for VorgangManager to finish search index initialisation."
until waitForInitSearchIndex
do
	echo -n "."
    sleep 1
done
echo

echo
echo "Starting VorgangManager, Alfa-Server for normal operations."

$DOCKER_COMPOSE_CMD up -d vorgang-manager alfa

waitForServerStarted() {
	($DOCKER_COMPOSE_CMD logs | awk 'BEGIN{RS="\0"} /Started AlfaServerApplication/ && /Started VorgangManagerServerApplication/ { exit 1 }')
}

echo
echo "Waiting for VorgangManager and Alfa Server to log successful start."
while waitForServerStarted
do
	echo -n "."
    sleep 1
done
echo

echo "done."