/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { Component, HostBinding } from '@angular/core';
import { CommonModule } from '@angular/common';
import { Dialog, DialogModule, DialogRef, CdkDialogContainer } from '@angular/cdk/dialog';
import { PortalModule } from '@angular/cdk/portal';
import { BescheidPaperComponent } from '../bescheid-paper/bescheid-paper.component';
import { BescheidStepperComponent } from '../bescheid-stepper/bescheid-stepper.component';

@Component({
  imports: [DialogModule, PortalModule],
  standalone: true,
  template: '<ng-template cdkPortalOutlet></ng-template>',
})
export class BescheidDialogContainerComponent extends CdkDialogContainer {}

@Component({
  selector: 'app-bescheid-dialog',
  standalone: true,
  imports: [CommonModule, DialogModule, BescheidPaperComponent, BescheidStepperComponent],
  templateUrl: './bescheid-dialog.component.html',
})
export class BescheidDialogComponent {
  constructor(public dialogRef: DialogRef) {}
}

@Component({
  selector: 'app-bescheid-dialog-button',
  templateUrl: 'bescheid-dialog-button.html',
  standalone: true,
  imports: [DialogModule],
})
export class BescheidDialogExampleComponent {
  constructor(private readonly cdkDialog: Dialog) {}

  openDialog(): void {
    this.cdkDialog
      .open<string>(BescheidDialogComponent, {
        container: BescheidDialogContainerComponent,
        hasBackdrop: true,
        backdropClass: 'bescheid-dialog-backdrop',
      })
      .closed.subscribe((result) => {
        console.log('The dialog was closed', result);
      });
  }
}
