import { ConfigurationLinkRel } from '@admin-client/configuration-shared';
import { createEmptyStateResource, createStateResource, HasLinkPipe } from '@alfa-client/tech-shared';
import { existsAsHtmlElement, notExistsAsHtmlElement } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { IconComponent, NavItemComponent } from '@ods/system';
import { createConfigurationResource } from 'libs/admin/configuration-shared/test/configuration';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { StatistikMenuComponent } from './statistik-menu.component';

describe('StatistikMenuComponent', () => {
  let component: StatistikMenuComponent;
  let fixture: ComponentFixture<StatistikMenuComponent>;

  const auswertungenNavigationSelector: string = getDataTestIdOf('auswertungen-navigation');
  const datenfreigabenNavigationSelector: string = getDataTestIdOf('datenfreigaben-navigation');
  const datenanfragenNavigationSelector: string = getDataTestIdOf('datenanfragen-navigation');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      imports: [StatistikMenuComponent, HasLinkPipe, MockComponent(NavItemComponent), MockComponent(IconComponent)],
    }).compileComponents();

    fixture = TestBed.createComponent(StatistikMenuComponent);
    component = fixture.componentInstance;
    component.configurationStateResource = createEmptyStateResource();
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('statistic navigation', () => {
    describe('auswertungen', () => {
      it('should show if agregation mapping link is present', () => {
        component.configurationStateResource = createStateResource(
          createConfigurationResource([ConfigurationLinkRel.AUSWERTUNG]),
        );

        fixture.detectChanges();

        existsAsHtmlElement(fixture, auswertungenNavigationSelector);
      });

      it('should hide if agregation mapping link is missing', () => {
        component.configurationStateResource = createStateResource(createConfigurationResource());

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, auswertungenNavigationSelector);
      });
    });

    describe('datenfreigaben', () => {
      it('should show if datenfreigabe link is present', () => {
        component.configurationStateResource = createStateResource(
          createConfigurationResource([ConfigurationLinkRel.DATENFREIGABE]),
        );

        fixture.detectChanges();

        existsAsHtmlElement(fixture, datenfreigabenNavigationSelector);
      });

      it('should hide if datenfreigabe link is missing', () => {
        component.configurationStateResource = createStateResource(createConfigurationResource());

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, datenfreigabenNavigationSelector);
      });
    });

    describe('datenanfragen', () => {
      it('should show if datenanfrage link is present', () => {
        component.configurationStateResource = createStateResource(
          createConfigurationResource([ConfigurationLinkRel.DATENANFRAGE]),
        );

        fixture.detectChanges();

        existsAsHtmlElement(fixture, datenanfragenNavigationSelector);
      });

      it('should hide if datenanfrage link is missing', () => {
        component.configurationStateResource = createStateResource(createConfigurationResource());

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, datenanfragenNavigationSelector);
      });
    });
  });
});
