import { NavigationService, RouteData } from '@alfa-client/navigation-shared';
import {
  ApiResourceService,
  createEmptyStateResource,
  createStateResource,
  decodeUrlFromEmbedding,
  ResourceRepository,
  ResourceServiceConfig,
  StateResource,
  StateService,
} from '@alfa-client/tech-shared';
import { UrlSegment } from '@angular/router';
import { iif, map, Observable, of, switchMap } from 'rxjs';
import * as self from './auswertung-resource.service';
import { AuswertungLinkRel } from './auswertung.linkrel';
import { AuswertungResource } from './auswertung.model';
import { AUSWERTUNG_FEATURE_KEY, AUSWERTUNG_PATH } from './auswertung.state';

export class AuswertungResourceService extends ApiResourceService<AuswertungResource, AuswertungResource> {}

export function createAuswertungResourceService(
  repository: ResourceRepository,
  stateService: StateService,
  navigationService: NavigationService,
): AuswertungResourceService {
  return new AuswertungResourceService(_buildResourceServiceConfig(repository, navigationService), stateService, repository);
}

export function _buildResourceServiceConfig(
  repository: ResourceRepository,
  navigationService: NavigationService,
): ResourceServiceConfig<AuswertungResource> {
  return {
    stateInfo: { name: AUSWERTUNG_FEATURE_KEY, path: AUSWERTUNG_PATH },
    baseResource: self._getResourceByNavigationRoute(repository, navigationService),
    getLinkRel: AuswertungLinkRel.SELF,
    edit: { linkRel: AuswertungLinkRel.SELF },
    delete: { linkRel: AuswertungLinkRel.SELF },
  };
}

export function _getResourceByNavigationRoute(
  repository: ResourceRepository,
  navigationService: NavigationService,
): Observable<StateResource<AuswertungResource>> {
  return navigationService
    .getCurrentRouteData()
    .pipe(
      switchMap((route: RouteData) =>
        iif(
          () => self._isAuswertungEditUrl(route),
          self._getAuswertungResourceByRoute(repository, route),
          of(createEmptyStateResource<AuswertungResource>()),
        ),
      ),
    );
}

export function _isAuswertungEditUrl(route: RouteData): boolean {
  const urlSegments: UrlSegment[] = route.urlSegments;
  return (
    urlSegments.length === 3 &&
    urlSegments[0].path === 'statistik' &&
    urlSegments[1].path === 'auswertungen' &&
    urlSegments[2].path !== 'neu'
  );
}

export function _getAuswertungResourceByRoute(
  repository: ResourceRepository,
  route: RouteData,
): Observable<StateResource<AuswertungResource>> {
  const uri: string = decodeUrlFromEmbedding(route.urlSegments[2].path);
  return repository.getResource(uri).pipe(map((resource: AuswertungResource) => createStateResource(resource)));
}
