import { createStateResource, StateResource } from '@alfa-client/tech-shared';
import { Mock, mock } from '@alfa-client/test-utils';
import { TestBed } from '@angular/core/testing';
import { expect } from '@jest/globals';
import { singleCold, singleColdCompleted } from 'libs/tech-shared/test/marbles';
import { Observable, of } from 'rxjs';
import { createAuswertung, createAuswertungListResource, createAuswertungResource } from '../../../test/auswertung';
import { AuswertungListResourceService } from './auswertung-list-resource.service';
import { AuswertungResourceService } from './auswertung-resource.service';
import { Auswertung, AuswertungListResource, AuswertungResource } from './auswertung.model';
import { AuswertungService } from './auswertung.service';

describe('AuswertungService', () => {
  let service: AuswertungService;
  let listResourceService: Mock<AuswertungListResourceService>;
  let resourceService: Mock<AuswertungResourceService>;

  beforeEach(() => {
    listResourceService = mock(AuswertungListResourceService);
    resourceService = mock(AuswertungResourceService);

    TestBed.configureTestingModule({
      providers: [
        AuswertungService,
        { provide: AuswertungListResourceService, useValue: listResourceService },
        { provide: AuswertungResourceService, useValue: resourceService },
      ],
    });

    service = TestBed.inject(AuswertungService);
  });

  it('should create', () => {
    expect(service).toBeTruthy();
  });

  describe('get list', () => {
    const auswertungListResource: AuswertungListResource = createAuswertungListResource();
    const auswertungListStateResource: StateResource<AuswertungListResource> = createStateResource(auswertungListResource);

    beforeEach(() => {
      listResourceService.getList = jest.fn().mockReturnValue(singleCold(auswertungListStateResource));
    });

    it('should call listResourceService', () => {
      service.getList();

      expect(listResourceService.getList).toHaveBeenCalled();
    });

    it('should return value', () => {
      const loadedAuswertungListResource: Observable<StateResource<AuswertungListResource>> = service.getList();

      expect(loadedAuswertungListResource).toBeObservable(singleCold(auswertungListStateResource));
    });
  });

  describe('create', () => {
    const auswertungResource: AuswertungResource = createAuswertungResource();
    const auswertungStateResource: StateResource<AuswertungResource> = createStateResource(auswertungResource);

    const auswertung: Auswertung = createAuswertung();

    beforeEach(() => {
      listResourceService.create = jest.fn().mockReturnValue(singleCold(auswertungStateResource));
    });

    it('should call resourceService', () => {
      service.create(auswertung);

      expect(listResourceService.create).toHaveBeenCalledWith(auswertung);
    });

    it('should return value', () => {
      const loadedAuswertungResource: Observable<StateResource<AuswertungResource>> = service.create(auswertung);

      expect(loadedAuswertungResource).toBeObservable(singleCold(auswertungStateResource));
    });
  });

  describe('refresh list', () => {
    it('should call list resource service', () => {
      listResourceService.refresh = jest.fn();

      service.refreshList();

      expect(listResourceService.refresh).toHaveBeenCalled();
    });
  });

  describe('get', () => {
    const stateResource: StateResource<AuswertungResource> = createStateResource(createAuswertungResource());

    beforeEach(() => {
      resourceService.get = jest.fn().mockReturnValue(of(stateResource));
    });

    it('should call resource service', () => {
      service.get();

      expect(resourceService.get).toHaveBeenCalled();
    });

    it('should emit resource', () => {
      const stateResource$: Observable<StateResource<AuswertungResource>> = service.get();

      expect(stateResource$).toBeObservable(singleColdCompleted(stateResource));
    });
  });

  describe('save', () => {
    const auswertung: Auswertung = createAuswertung();
    const stateResource: StateResource<AuswertungResource> = createStateResource(createAuswertungResource());
    beforeEach(() => {
      resourceService.save = jest.fn().mockReturnValue(of(stateResource));
    });

    it('should call resource service', () => {
      service.save(auswertung);

      expect(resourceService.save).toHaveBeenCalledWith(auswertung);
    });

    it('should emit saved state resource', () => {
      const saved$: Observable<StateResource<AuswertungResource>> = service.save(auswertung);

      expect(saved$).toBeObservable(singleColdCompleted(stateResource));
    });
  });
});
