import { ADMIN_FORMSERVICE } from '@admin-client/shared';
import { EMPTY_STRING } from '@alfa-client/tech-shared';
import { existsAsHtmlElement, getElementComponentFromFixtureByCss, mock, Mock, mockGetValue } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormBuilder, FormControl, FormGroup, ReactiveFormsModule } from '@angular/forms';
import { expect } from '@jest/globals';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { AuswertungFormservice } from '../auswertung.formservice';
import { AuswertungFieldFormComponent } from './auswertung-field-form/auswertung-field-form.component';
import { AuswertungFieldListFormComponent } from './auswertung-field-list-form.component';

describe('AuswertungFieldListFormComponent', () => {
  let component: AuswertungFieldListFormComponent;
  let fixture: ComponentFixture<AuswertungFieldListFormComponent>;

  const fieldForm: string = getDataTestIdOf('auswertung-field-mapping-form-0');

  const formBuilder: FormBuilder = new FormBuilder();

  let formService: Mock<AuswertungFormservice>;

  beforeEach(async () => {
    const form: FormGroup = formBuilder.group({
      [AuswertungFormservice.FIELD_MAPPINGS]: formBuilder.array([
        new FormGroup({
          [AuswertungFormservice.FIELD_MAPPING_SOURCE_PATH]: new FormControl(EMPTY_STRING),
          [AuswertungFormservice.FIELD_MAPPING_TARGET_PATH]: new FormControl(EMPTY_STRING),
        }),
      ]),
    });

    formService = <any>{
      ...mock(AuswertungFormservice),
      form,
      addMapping: jest.fn(),
      removeMapping: jest.fn(),
    };

    mockGetValue(formService, AuswertungFormservice.FIELD_MAPPINGS, form.controls[AuswertungFormservice.FIELD_MAPPINGS]);

    await TestBed.configureTestingModule({
      imports: [ReactiveFormsModule, AuswertungFieldListFormComponent, MockComponent(AuswertungFieldFormComponent)],
      providers: [
        {
          provide: ADMIN_FORMSERVICE,
          useValue: formService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(AuswertungFieldListFormComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('template', () => {
    describe('mapping input', () => {
      it('should exists', () => {
        fixture.detectChanges();

        existsAsHtmlElement(fixture, fieldForm);
      });

      it('should have inputs', () => {
        const fieldFormComponent: AuswertungFieldFormComponent = getElementComponentFromFixtureByCss(fixture, fieldForm);

        expect(fieldFormComponent.index).toEqual(0);
      });
    });
  });
});
