import { AuswertungResource } from '@admin-client/reporting-shared';
import { ADMIN_FORMSERVICE, AdminCancelButtonComponent, AdminSaveButtonComponent, ROUTES } from '@admin-client/shared';
import { createStateResource, EMPTY_STRING, StateResource } from '@alfa-client/tech-shared';
import {
  dispatchEventFromFixture,
  existsAsHtmlElement,
  getElementFromFixtureByType,
  mock,
  Mock,
  MockEvent,
} from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormBuilder, FormControl, FormGroup, ReactiveFormsModule } from '@angular/forms';
import { expect } from '@jest/globals';
import { TextEditorComponent } from '@ods/component';
import { ButtonComponent, IconComponent } from '@ods/system';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { getDataTestIdOf } from '../../../../../../../tech-shared/test/data-test';
import { singleColdCompleted } from '../../../../../../../tech-shared/test/marbles';
import { createAuswertungResource } from '../../../../../../reporting-shared/test/auswertung';
import { AuswertungFieldListFormComponent } from './auswertung-field-list-form/auswertung-field-list-form.component';
import { AuswertungFormComponent } from './auswertung-form.component';
import { AuswertungFormservice } from './auswertung.formservice';

describe('AuswertungFormComponent', () => {
  let component: AuswertungFormComponent;
  let fixture: ComponentFixture<AuswertungFormComponent>;

  const formEngineNameInputTestId: string = getDataTestIdOf('form-engine-name-text-editor');
  const formIdInputTestId: string = getDataTestIdOf('form-id-text-editor');
  const addMappingButton: string = getDataTestIdOf('add-mapping');

  const auswertungStateResource: StateResource<AuswertungResource> = createStateResource(createAuswertungResource());

  const formBuilder: FormBuilder = new FormBuilder();

  let formService: Mock<AuswertungFormservice>;

  beforeEach(async () => {
    const form: FormGroup = formBuilder.group({
      [AuswertungFormservice.FIELD_NAME]: new FormControl(EMPTY_STRING),
      [AuswertungFormservice.FIELD_FORM_IDENTIFIER]: formBuilder.group({
        [AuswertungFormservice.FIELD_FORM_ENGINE_NAME]: new FormControl(EMPTY_STRING),
        [AuswertungFormservice.FIELD_FORM_ID]: new FormControl(EMPTY_STRING),
      }),
    });

    formService = <any>{ ...mock(AuswertungFormservice), form };
    formService.get = jest.fn().mockReturnValue(of(auswertungStateResource));

    await TestBed.configureTestingModule({
      imports: [
        ReactiveFormsModule,
        AuswertungFormComponent,
        MockComponent(TextEditorComponent),
        MockComponent(ButtonComponent),
        MockComponent(IconComponent),
        MockComponent(AdminSaveButtonComponent),
        MockComponent(AdminCancelButtonComponent),
        MockComponent(AuswertungFieldListFormComponent),
      ],
    })
      .overrideComponent(AuswertungFormComponent, {
        set: {
          providers: [
            {
              provide: AuswertungFormservice,
              useValue: formService,
            },
            {
              provide: ADMIN_FORMSERVICE,
              useValue: formService,
            },
          ],
        },
      })
      .compileComponents();

    fixture = TestBed.createComponent(AuswertungFormComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('component', () => {
    it('should get auswertung from form service', () => {
      expect(formService.get).toHaveBeenCalled();
    });

    it('should initial values', () => {
      expect(component.auswertungStateResource$).toBeObservable(singleColdCompleted(auswertungStateResource));
    });
  });

  describe('template', () => {
    describe('form engine input', () => {
      it('should exists', () => {
        fixture.detectChanges();

        existsAsHtmlElement(fixture, formEngineNameInputTestId);
      });
    });

    describe('form id input', () => {
      it('should exists', () => {
        fixture.detectChanges();

        existsAsHtmlElement(fixture, formIdInputTestId);
      });
    });

    describe('add mapping button', () => {
      it('should exists', () => {
        fixture.detectChanges();

        existsAsHtmlElement(fixture, addMappingButton);
      });

      describe('output', () => {
        describe('clickEmitter', () => {
          it('should call formService', () => {
            fixture.detectChanges();

            dispatchEventFromFixture(fixture, addMappingButton, MockEvent.CLICK);

            expect(formService.addMapping).toHaveBeenCalled();
          });
        });
      });
    });

    describe('save button', () => {
      it('should have link path', () => {
        const comp: AdminSaveButtonComponent = getElementFromFixtureByType(fixture, AdminSaveButtonComponent);

        expect(comp.successLinkPath).toEqual(ROUTES.AUSWERTUNGEN);
      });
    });

    describe('cancel button', () => {
      it('should have link path', () => {
        const comp: AdminCancelButtonComponent = getElementFromFixtureByType(fixture, AdminCancelButtonComponent);

        expect(comp.linkPath).toEqual(ROUTES.AUSWERTUNGEN);
      });
    });
  });
});
