/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { AuswertungListResource, AuswertungService } from '@admin-client/reporting-shared';
import { createStateResource, StateResource } from '@alfa-client/tech-shared';
import { expectComponentExistsInTemplate, getElementFromFixtureByType, mock, Mock } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { expect } from '@jest/globals';
import { RoutingButtonComponent } from '@ods/component';
import { singleCold } from 'libs/tech-shared/test/marbles';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { createAuswertungListResource } from '../../../../../reporting-shared/test/auswertung';
import { AuswertungListContainerComponent } from './auswertung-list-container.component';
import { AuswertungListComponent } from './auswertung-list/auswertung-list.component';

describe('AuswertungListContainerComponent', () => {
  let component: AuswertungListContainerComponent;
  let fixture: ComponentFixture<AuswertungListContainerComponent>;

  let auswertungService: Mock<AuswertungService>;

  beforeEach(async () => {
    auswertungService = mock(AuswertungService);

    await TestBed.configureTestingModule({
      imports: [AuswertungListContainerComponent, MockComponent(RoutingButtonComponent), MockComponent(AuswertungListComponent)],
      providers: [
        {
          provide: AuswertungService,
          useValue: auswertungService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(AuswertungListContainerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('on init', () => {
    const stateResource: StateResource<AuswertungListResource> = createStateResource(createAuswertungListResource());

    beforeEach(() => {
      auswertungService.getList.mockReturnValue(singleCold(stateResource));
    });

    it('should call service to get list', () => {
      component.ngOnInit();

      expect(auswertungService.getList).toHaveBeenCalled();
    });

    it('should assign stateResource', () => {
      component.ngOnInit();

      expect(component.listStateResource$).toBeObservable(singleCold(stateResource));
    });
  });

  describe('on destroy', () => {
    it('should refresh auswertung list', () => {
      component.ngOnDestroy();

      expect(auswertungService.refreshList).toHaveBeenCalled();
    });
  });

  describe('template', () => {
    describe('auswertung list', () => {
      it('should exists', () => {
        expectComponentExistsInTemplate(fixture, AuswertungListComponent);
      });

      it('should have inputs', () => {
        const stateResource: StateResource<AuswertungListResource> = createStateResource(createAuswertungListResource());
        component.listStateResource$ = of(stateResource);
        fixture.detectChanges();

        const comp: AuswertungListComponent = getElementFromFixtureByType(fixture, AuswertungListComponent);

        expect(comp.auswertungListStateResource).toEqual(stateResource);
      });
    });
  });
});
