import { createEmptyStateResource, StateResource } from '@alfa-client/tech-shared';
import { dispatchEventFromFixture, existsAsHtmlElement, getMockComponent, MockEvent } from '@alfa-client/test-utils';
import { CancelDialogButtonComponent } from '@alfa-client/ui-shared';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ButtonWithSpinnerComponent } from '@ods/component';
import { MockComponent } from 'ng-mocks';
import { getDataTestIdOf } from '../../../../../../../tech-shared/test/data-test';
import { UserDeleteDialogComponent } from './user-delete-dialog.component';

describe('UserFormDeleteDialogComponent', () => {
  let component: UserDeleteDialogComponent;
  let fixture: ComponentFixture<UserDeleteDialogComponent>;

  const deleteButton: string = getDataTestIdOf('dialog-delete-button-host');
  const cancelButton: string = getDataTestIdOf('dialog-cancel-button-host');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      imports: [UserDeleteDialogComponent],
      declarations: [MockComponent(CancelDialogButtonComponent), MockComponent(ButtonWithSpinnerComponent)],
    }).compileComponents();

    fixture = TestBed.createComponent(UserDeleteDialogComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('template', () => {
    describe('cancel button', () => {
      it('should exist', () => {
        existsAsHtmlElement(fixture, cancelButton);
      });
    });

    describe('delete button', () => {
      it('should exist', () => {
        const stateResource: StateResource<unknown> = createEmptyStateResource();
        component.deleteUserStateResource = stateResource;

        fixture.detectChanges();

        expect(getMockComponent(fixture, ButtonWithSpinnerComponent).stateResource).toBe(stateResource);
      });

      it('should emit delete on click', () => {
        component.delete.emit = jest.fn();

        dispatchEventFromFixture(fixture, deleteButton, MockEvent.CLICK);

        expect(component.delete.emit).toHaveBeenCalled();
      });
    });
  });
});
