import { CommandListResourceService, CommandListResourceServiceConfig, CommandService } from '@alfa-client/command-shared';
import { ResourceListService, ResourceRepository } from '@alfa-client/tech-shared';
import { VorgangService, VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { BescheidListLinkRel } from './bescheid.linkrel';
import { BescheidListResource, BescheidResource } from './bescheid.model';
import { BESCHEID_FEATURE_KEY, BESCHEID_LIST_PATH } from './bescheid.state';

export class BescheidListResourceService extends CommandListResourceService<
  VorgangWithEingangResource,
  BescheidListResource,
  BescheidResource
> {}

export function createBescheidListResourceService(
  vorgangService: VorgangService,
  repository: ResourceRepository<BescheidResource>,
  commandService: CommandService,
): ResourceListService<VorgangWithEingangResource, BescheidListResource, BescheidResource> {
  return new BescheidListResourceService(buildBescheidListServiceConfig(vorgangService), repository, commandService);
}

function buildBescheidListServiceConfig(
  vorgangService: VorgangService,
): CommandListResourceServiceConfig<VorgangWithEingangResource> {
  return {
    stateInfo: { name: BESCHEID_FEATURE_KEY, path: BESCHEID_LIST_PATH },
    baseResource: vorgangService.selectVorgangWithEingang(),
    getLinkRel: VorgangWithEingangLinkRel.BESCHEIDE,
    listResourceListLinkRel: BescheidListLinkRel.BESCHEID_LIST,
  };
}
