import { CommandOrder, CommandResourceService, CommandService } from '@alfa-client/command-shared';
import { ResourceServiceConfig, StateService } from '@alfa-client/tech-shared';
import { VorgangService, VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { BescheidLinkRel } from './bescheid.linkrel';
import { BescheidResource } from './bescheid.model';
import { BESCHEID_DRAFT_PATH, BESCHEID_FEATURE_KEY } from './bescheid.state';

export class BescheidResourceService extends CommandResourceService<VorgangWithEingangResource, BescheidResource> {}

export function createBescheidResourceService(
  stateService: StateService,
  commandService: CommandService,
  vorgangService: VorgangService,
): CommandResourceService<VorgangWithEingangResource, BescheidResource> {
  console.info('Create Bescheid Resource Service');
  return new CommandResourceService(buildConfig(vorgangService), stateService, commandService);
}

function buildConfig(vorgangService: VorgangService): ResourceServiceConfig<VorgangWithEingangResource> {
  return {
    stateInfo: { name: BESCHEID_FEATURE_KEY, path: BESCHEID_DRAFT_PATH },
    baseResource: vorgangService.selectVorgangWithEingang(),
    getLinkRel: VorgangWithEingangLinkRel.BESCHEID_DRAFT,
    delete: { linkRel: BescheidLinkRel.DELETE, order: CommandOrder.DELETE_BESCHEID },
    edit: { linkRel: BescheidLinkRel.UPDATE, order: CommandOrder.UPDATE_BESCHEID },
  };
}
