/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BescheidResource } from '@alfa-client/bescheid-shared';
import {
  createDialogRefMock,
  DialogRefMock,
  existsAsHtmlElement,
  mock,
  Mock,
  triggerEvent,
  useFromMock,
} from '@alfa-client/test-utils';
import { createDialogResult, OzgcloudDialogCommandResult, OzgcloudDialogService } from '@alfa-client/ui-shared';
import { VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { EventEmitter } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { of } from 'rxjs';
import { createBescheidResource } from '../../../../../../../bescheid-shared/src/test/bescheid';
import { createSuccessfullyDoneCommandStateResource } from '../../../../../../../command-shared/test/command';
import { getDataTestIdOf } from '../../../../../../../tech-shared/test/data-test';
import { createVorgangWithEingangResource } from '../../../../../../../vorgang-shared/test/vorgang';
import {
  AbschliessenDialogData,
  BescheidWizardAbschliessenDialogContainerComponent,
} from '../abschliessen-dialog-container/bescheid-wizard-abschliessen-dialog-container.component';
import { BescheidWizardAbschliessenButtonComponent } from './bescheid-wizard-abschliessen-button.component';

describe('BescheidWizardAbschliessenButtonComponent', () => {
  let component: BescheidWizardAbschliessenButtonComponent;
  let fixture: ComponentFixture<BescheidWizardAbschliessenButtonComponent>;

  const abschliessenButton: string = getDataTestIdOf('bescheid-ueberspringen');

  let ozgcloudDialogService: Mock<OzgcloudDialogService>;

  beforeEach(() => {
    ozgcloudDialogService = mock(OzgcloudDialogService);
  });

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [BescheidWizardAbschliessenButtonComponent],
      providers: [
        {
          provide: OzgcloudDialogService,
          useValue: ozgcloudDialogService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(BescheidWizardAbschliessenButtonComponent);
    component = fixture.componentInstance;
    component.vorgangAbgeschlossen = useFromMock(mock(EventEmitter<void>));
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('component', () => {
    describe('onClick', () => {
      beforeEach(() => {
        component.handleDialogClosed = jest.fn();
      });

      it('should open dialog', () => {
        const vorgangWithEingangResource: VorgangWithEingangResource = createVorgangWithEingangResource();
        const bescheidResource: BescheidResource = createBescheidResource();
        component.vorgangWithEingangResource = vorgangWithEingangResource;
        component.bescheidResource = bescheidResource;

        component.onClick();

        expect(ozgcloudDialogService.openInCallingComponentContext).toHaveBeenCalledWith(
          BescheidWizardAbschliessenDialogContainerComponent,
          component.viewContainerRef,
          { vorgangWithEingangResource, bescheidResource } as AbschliessenDialogData,
        );
      });

      it('should handle dialog closed', () => {
        const dialogRef: DialogRefMock<OzgcloudDialogCommandResult> = createDialogRefMock();
        ozgcloudDialogService.openInCallingComponentContext.mockReturnValue(dialogRef);

        component.onClick();

        expect(component.handleDialogClosed).toHaveBeenCalledWith(dialogRef);
      });
    });
  });

  describe('handleDialogClosed', () => {
    let dialogRef: DialogRefMock<OzgcloudDialogCommandResult>;

    beforeEach(() => {
      dialogRef = createDialogRefMock();
    });

    it('should NOT emit vorgang abgeschlossen', () => {
      dialogRef.closed = of(undefined);

      component.handleDialogClosed(dialogRef as never);

      expect(component.vorgangAbgeschlossen.emit).not.toHaveBeenCalled();
    });

    it('should emit vorgang abgeschlossen', () => {
      dialogRef.closed = of(createDialogResult(createSuccessfullyDoneCommandStateResource()));

      component.handleDialogClosed(dialogRef as never);

      expect(component.vorgangAbgeschlossen.emit).toHaveBeenCalled();
    });
  });

  describe('template', () => {
    it('should have button', () => {
      existsAsHtmlElement(fixture, abschliessenButton);
    });

    describe('output', () => {
      it('should call onClick', () => {
        component.onClick = jest.fn();

        triggerEvent({ fixture, name: 'click', elementSelector: abschliessenButton });

        expect(component.onClick).toHaveBeenCalled();
      });
    });
  });
});
