/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BescheidResource } from '@alfa-client/bescheid-shared';
import { isDialogSuccessfullyCompleted, OzgcloudDialogCommandResult, OzgcloudDialogService } from '@alfa-client/ui-shared';
import { VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { DialogRef } from '@angular/cdk/dialog';
import { Component, EventEmitter, inject, Input, Output, ViewContainerRef } from '@angular/core';
import { filter } from 'rxjs';
import {
  AbschliessenDialogData,
  BescheidWizardAbschliessenDialogContainerComponent,
} from '../abschliessen-dialog-container/bescheid-wizard-abschliessen-dialog-container.component';

@Component({
  selector: 'alfa-bescheid-wizard-abschliessen-button',
  templateUrl: './bescheid-wizard-abschliessen-button.component.html',
  styles: [':host {@apply flex-1 flex items-end}'],
})
export class BescheidWizardAbschliessenButtonComponent {
  @Input() vorgangWithEingangResource: VorgangWithEingangResource;
  @Input() bescheidResource: BescheidResource;

  @Output() vorgangAbgeschlossen: EventEmitter<void> = new EventEmitter<void>();

  private readonly ozgcloudDialogService = inject(OzgcloudDialogService);
  readonly viewContainerRef = inject(ViewContainerRef);

  public onClick(): void {
    const dialogData: AbschliessenDialogData = {
      vorgangWithEingangResource: this.vorgangWithEingangResource,
      bescheidResource: this.bescheidResource,
    };

    const dialogRef: DialogRef<OzgcloudDialogCommandResult> = this.ozgcloudDialogService.openInCallingComponentContext(
      BescheidWizardAbschliessenDialogContainerComponent,
      this.viewContainerRef,
      dialogData,
    );

    this.handleDialogClosed(dialogRef);
  }

  handleDialogClosed(dialogRef: DialogRef<OzgcloudDialogCommandResult>): void {
    dialogRef.closed.pipe(filter(isDialogSuccessfullyCompleted)).subscribe(() => this.vorgangAbgeschlossen.emit());
  }
}
