import { BescheidDocument, BescheidLinkRel, createEmptyBescheidDocument } from '@alfa-client/bescheid-shared';
import { HasLinkPipe } from '@alfa-client/tech-shared';
import { existsAsHtmlElement, getElementComponentFromFixtureByCss, mock, Mock, triggerEvent } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ButtonCardComponent, IconComponent } from '@ods/system';
import { MockComponent } from 'ng-mocks';
import { createBescheidDocument, createBescheidResource } from '../../../../../../../bescheid-shared/src/test/bescheid';
import { getDataTestIdOf } from '../../../../../../../tech-shared/test/data-test';
import { BescheidFormService } from '../../../bescheid.formservice';
import { BescheidWizardCreateDocumentButtonComponent } from './bescheid-wizard-create-document-button.component';

describe('BescheidWizardAutomatischErstellenButtonComponent', () => {
  let component: BescheidWizardCreateDocumentButtonComponent;
  let fixture: ComponentFixture<BescheidWizardCreateDocumentButtonComponent>;

  const createButtonTestId: string = getDataTestIdOf('create-bescheid-document-button');

  let formService: Mock<BescheidFormService>;

  const bescheidDocument: BescheidDocument = createBescheidDocument();

  beforeEach(() => {
    formService = mock(BescheidFormService);
  });

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        BescheidWizardCreateDocumentButtonComponent,
        MockComponent(ButtonCardComponent),
        MockComponent(IconComponent),
        HasLinkPipe,
      ],
      providers: [{ provide: BescheidFormService, useValue: formService }],
    }).compileComponents();

    fixture = TestBed.createComponent(BescheidWizardCreateDocumentButtonComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  describe('component', () => {
    it('should create', () => {
      expect(component).toBeTruthy();
    });

    it('should set initial values', () => {
      expect(component.bescheidDocument).toEqual(createEmptyBescheidDocument());
    });

    describe('set bescheid document', () => {
      it('should set value', () => {
        component.bescheidDocument = bescheidDocument;

        expect(component.bescheidDocument).toEqual(bescheidDocument);
      });

      it('should call form service', () => {
        component.bescheidDocument = bescheidDocument;

        expect(formService.updateBescheidDocumentFile).toHaveBeenCalledWith(bescheidDocument.documentUri);
      });
    });
  });

  describe('template', () => {
    describe('create button', () => {
      it('should exists', () => {
        fixture.detectChanges();

        existsAsHtmlElement(fixture, createButtonTestId);
      });

      it('should have been called with inputs', () => {
        component.bescheidResource = createBescheidResource([BescheidLinkRel.CREATE_DOCUMENT]);
        component.bescheidDocument = bescheidDocument;

        fixture.detectChanges();
        const buttonComponent: ButtonCardComponent = getElementComponentFromFixtureByCss<ButtonCardComponent>(
          fixture,
          createButtonTestId,
        );

        expect(buttonComponent.isLoading).toEqual(bescheidDocument.create.loading);
      });

      describe('output', () => {
        describe('clickEmitter', () => {
          it('should emit', () => {
            component.clickEmitter.emit = jest.fn();
            component.bescheidResource = createBescheidResource([BescheidLinkRel.CREATE_DOCUMENT]);
            fixture.detectChanges();

            triggerEvent({ fixture, elementSelector: createButtonTestId, name: 'click' });

            expect(component.clickEmitter.emit).toHaveBeenCalled();
          });
        });
      });
    });
  });
});
