import { BescheidDocument, BescheidLinkRel, BescheidService, createEmptyUploadInProgress } from '@alfa-client/bescheid-shared';
import { HasLinkPipe } from '@alfa-client/tech-shared';
import {
  existsAsHtmlElement,
  getElementComponentFromFixtureByCss,
  mock,
  triggerEvent,
  useFromMock,
} from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ReactiveFormsModule, UntypedFormBuilder } from '@angular/forms';
import { SingleFileUploadEditorComponent } from '@ods/component';
import { IconComponent, SpinnerIconComponent } from '@ods/system';
import { MockComponent } from 'ng-mocks';
import {
  createBescheidDocument,
  createBescheidResource,
  createPendingUpload,
} from '../../../../../../../bescheid-shared/src/test/bescheid';
import { getDataTestIdOf } from '../../../../../../../tech-shared/test/data-test';
import { createFile } from '../../../../../../../tech-shared/test/file';
import { BescheidFormService } from '../../../bescheid.formservice';
import { BescheidWizardUploadDocumentButtonComponent } from './bescheid-wizard-upload-document-button.component';

describe('BescheidWizardDokumentHochladenButtonComponent', () => {
  let component: BescheidWizardUploadDocumentButtonComponent;
  let fixture: ComponentFixture<BescheidWizardUploadDocumentButtonComponent>;

  const fileUploadEditorTestId: string = getDataTestIdOf('bescheid-wizard-upload-document-upload-editor');

  const bescheidDocument: BescheidDocument = createBescheidDocument();

  let formService: BescheidFormService;

  beforeEach(() => {
    formService = new BescheidFormService(new UntypedFormBuilder(), useFromMock(mock(BescheidService)));
  });

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        BescheidWizardUploadDocumentButtonComponent,
        MockComponent(SingleFileUploadEditorComponent),
        MockComponent(IconComponent),
        MockComponent(SpinnerIconComponent),
        HasLinkPipe,
      ],
      providers: [
        {
          provide: BescheidFormService,
          useValue: formService,
        },
      ],
      imports: [ReactiveFormsModule],
    }).compileComponents();

    fixture = TestBed.createComponent(BescheidWizardUploadDocumentButtonComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  describe('component', () => {
    it('should create', () => {
      expect(component).toBeTruthy();
    });

    it('should set initial values', () => {
      expect(component.upload).toEqual(createEmptyUploadInProgress());
    });

    describe('set bescheid document', () => {
      it('should call form service', () => {
        formService.updateBescheidDocumentFile = jest.fn();

        component.bescheidDocument = bescheidDocument;

        expect(formService.updateBescheidDocumentFile).toHaveBeenCalledWith(bescheidDocument.documentUri);
      });

      it('should set upload', () => {
        formService.updateBescheidDocumentFile = jest.fn();

        component.bescheidDocument = bescheidDocument;

        expect(component.upload).toEqual(bescheidDocument.upload);
      });
    });

    describe('uploadFile', () => {
      it('should emit', () => {
        component.file.emit = jest.fn();
        const file: File = createFile();

        component.uploadFile(file);

        expect(component.file.emit).toHaveBeenCalledWith(file);
      });
    });
  });

  describe('template', () => {
    describe('file upload editor', () => {
      it('should exists', () => {
        component.bescheidResource = createBescheidResource([BescheidLinkRel.UPLOAD_BESCHEID_FILE]);

        fixture.detectChanges();

        existsAsHtmlElement(fixture, fileUploadEditorTestId);
      });

      it('should have been called with inputs', () => {
        component.bescheidResource = createBescheidResource([BescheidLinkRel.UPLOAD_BESCHEID_FILE]);
        component.upload = createPendingUpload();

        fixture.detectChanges();
        const fileUploadEditorComponent: SingleFileUploadEditorComponent =
          getElementComponentFromFixtureByCss<SingleFileUploadEditorComponent>(fixture, fileUploadEditorTestId);

        expect(fileUploadEditorComponent.uploadInProgress).toEqual(component.upload.loading);
      });

      describe('output', () => {
        describe('newFile', () => {
          it('should call handler', () => {
            component.bescheidResource = createBescheidResource([BescheidLinkRel.UPLOAD_BESCHEID_FILE]);
            const file: File = createFile();
            component.uploadFile = jest.fn();
            fixture.detectChanges();

            triggerEvent({ fixture, elementSelector: fileUploadEditorTestId, name: 'newFile', data: file });

            expect(component.uploadFile).toHaveBeenCalledWith(file);
          });
        });
      });
    });
  });
});
