/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BinaryFileListResource, BinaryFileResource } from '@alfa-client/binary-file-shared';
import { createStateResource, StateResource, ToEmbeddedResourcesPipe } from '@alfa-client/tech-shared';
import { existsAsHtmlElement, getMockComponent, notExistsAsHtmlElement } from '@alfa-client/test-utils';
import { SpinnerComponent } from '@alfa-client/ui';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { AttachmentWrapperComponent } from '@ods/system';
import { createBinaryFileListResource, createBinaryFileResource } from 'libs/binary-file-shared/test/binary-file';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent, MockDirective } from 'ng-mocks';
import { BinaryFile2ContainerComponent } from '../../binary-file2-container/binary-file2-container.component';
import { BinaryFileListOrientationDirective } from '../../directive/binary-file-list-orientation/binary-file-list-orientation.directive';
import { BinaryFileListComponent } from './binary-file-list.component';

describe('BinaryFileListComponent', () => {
  let component: BinaryFileListComponent;
  let fixture: ComponentFixture<BinaryFileListComponent>;

  const binaryFile: BinaryFileResource = createBinaryFileResource();
  function getBinaryFileListStateResource(binaryFiles: BinaryFileResource[]): StateResource<BinaryFileListResource> {
    return createStateResource(createBinaryFileListResource(binaryFiles));
  }

  const wrapperSelector: string = getDataTestIdOf('binary-file-list-wrapper');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        BinaryFileListComponent,
        ToEmbeddedResourcesPipe,
        MockComponent(BinaryFile2ContainerComponent),
        MockComponent(AttachmentWrapperComponent),
        MockComponent(SpinnerComponent),
        MockDirective(BinaryFileListOrientationDirective),
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(BinaryFileListComponent);
    component = fixture.componentInstance;
    component.binaryFileListStateResource = getBinaryFileListStateResource([binaryFile]);
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('template', () => {
    describe('attachment wrapper', () => {
      it('should show', () => {
        existsAsHtmlElement(fixture, wrapperSelector);
      });

      it('should hide', () => {
        component.binaryFileListStateResource = getBinaryFileListStateResource([]);

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, wrapperSelector);
      });
    });
  });

  describe('binary file container', () => {
    it('should be called with file', () => {
      const binaryFileContainerComponent: BinaryFile2ContainerComponent = getMockComponent<BinaryFile2ContainerComponent>(
        fixture,
        BinaryFile2ContainerComponent,
      );

      expect(binaryFileContainerComponent.file).toBe(binaryFile);
    });

    it('should be called with deleteable', () => {
      const binaryFileContainerComponent: BinaryFile2ContainerComponent = getMockComponent<BinaryFile2ContainerComponent>(
        fixture,
        BinaryFile2ContainerComponent,
      );

      expect(binaryFileContainerComponent.deletable).toBeFalsy();
    });
  });
});
