/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import {
  CommandListResourceService,
  CommandListResourceServiceConfig,
  CommandOrder,
  CommandService,
} from '@alfa-client/command-shared';
import { ResourceRepository } from '@alfa-client/tech-shared';
import { VorgangResource, VorgangService, VorgangWithEingangLinkRel } from '@alfa-client/vorgang-shared';
import { CollaborationListLinkRel } from './collaboration.linkrel';
import { CollaborationListResource, CollaborationResource } from './collaboration.model';

export class CollaborationListResourceService extends CommandListResourceService<
  VorgangResource,
  CollaborationListResource,
  CollaborationResource
> {}

export function createCollaborationListResourceService(
  vorgangService: VorgangService,
  repository: ResourceRepository<CollaborationResource>,
  commandService: CommandService,
) {
  return new CollaborationListResourceService(buildConfig(vorgangService), repository, commandService);
}

function buildConfig(vorgangService: VorgangService): CommandListResourceServiceConfig<VorgangResource> {
  return {
    baseResource: vorgangService.selectVorgangWithEingang(),
    getLinkRel: VorgangWithEingangLinkRel.COLLABORATIONS,
    listResourceListLinkRel: CollaborationListLinkRel.COLLABORATION_LIST,
    create: {
      linkRel: CollaborationListLinkRel.CREATE_COLLABORATION_REQUEST,
      order: CommandOrder.CREATE_COLLABORATION_REQUEST,
    },
  };
}
