import {
  createStateResource,
  LinkRelationName,
  ListItemResource,
  ListResource,
  ResourceRepository,
  StateResource,
} from '@alfa-client/tech-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { Resource } from '@ngxp/rest';
import { BehaviorSubject, of } from 'rxjs';
import { createDummy, DummyLinkRel, DummyListLinkRel } from '../../../tech-shared/test/dummy';
import { createDummyResource } from '../../../tech-shared/test/resource';
import { CommandListResourceService } from './command-list-resource.service';
import { CommandListResourceServiceConfig, CommandOrder } from './command.model';
import { CommandService } from './command.service';

describe('CommandListResourceService', () => {
  let service: CommandListResourceService<Resource, ListResource, ListItemResource>;
  let config: CommandListResourceServiceConfig<Resource>;
  let resourceRepository: Mock<ResourceRepository>;
  let commandService: Mock<CommandService>;

  const getLinkRel: LinkRelationName = DummyListLinkRel.LIST;
  const createLinkRel: LinkRelationName = DummyLinkRel.DUMMY;
  const createCommandOrder: CommandOrder = CommandOrder.CREATE_KOMMENTAR;
  const listResourceListLinkRel: LinkRelationName = DummyListLinkRel.LIST;

  const dummyCommandBody = createDummy();
  const dummyResource: Resource = createDummyResource();
  const dummyState: StateResource<Resource> = createStateResource(dummyResource);

  const baseResourceSubj: BehaviorSubject<StateResource<Resource>> = new BehaviorSubject<StateResource<Resource>>(dummyState);

  beforeEach(() => {
    config = {
      baseResource: baseResourceSubj,
      getLinkRel,
      listResourceListLinkRel,
      create: { linkRel: createLinkRel, order: createCommandOrder },
    };
    resourceRepository = mock(ResourceRepository);
    commandService = mock(CommandService);

    service = new CommandListResourceService(config, useFromMock(resourceRepository), useFromMock(commandService));
  });

  describe('createListItem', () => {
    it('should call createCommandByProp from commandService', () => {
      jest.spyOn(service as any, 'getListResourceAsObservable').mockReturnValue(of(dummyState));

      service.createListItem(dummyCommandBody).subscribe();

      expect(commandService.createCommandByProps).toHaveBeenCalledWith({
        resource: dummyResource,
        linkRel: createLinkRel,
        command: { order: createCommandOrder, body: dummyCommandBody },
        snackBarMessage: '',
      });
    });
  });
});
