import { ListItemResource, ListResource, ResourceListService, ResourceRepository, StateResource } from '@alfa-client/tech-shared';
import { Resource } from '@ngxp/rest';
import { Observable, first } from 'rxjs';
import { switchMap } from 'rxjs/operators';
import { CommandListResourceServiceConfig, CommandResource, CreateCommand } from './command.model';
import { CommandService } from './command.service';

export class CommandListResourceService<
  B extends Resource,
  T extends ListResource,
  I extends ListItemResource,
> extends ResourceListService<B, T, I> {
  constructor(
    protected config: CommandListResourceServiceConfig<B>,
    protected repository: ResourceRepository<I>,
    private commandService: CommandService,
  ) {
    super(config, repository);
  }

  public createListItem(toCreatelistItem: unknown): Observable<StateResource<CommandResource>> {
    return this.getListResourceAsObservable().pipe(
      first(),
      switchMap((listResource: StateResource<T>) =>
        this.createCommand(listResource.resource, this.config.create.linkRel, this.createCreateCommand(toCreatelistItem)),
      ),
    );
  }

  private createCommand(resource: Resource, linkRel: string, command: CreateCommand): Observable<StateResource<CommandResource>> {
    return this.commandService.createCommandByProps({
      resource,
      linkRel,
      command: command,
      snackBarMessage: '',
    });
  }

  private createCreateCommand(body: unknown): CreateCommand {
    return { order: this.config.create.order, body };
  }
}
