import { CommonModule } from '@angular/common';
import { Component } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormGroupDirective, ReactiveFormsModule, ValidationErrors } from '@angular/forms';
import { FormControlEditorAbstractComponent } from '@ods/component';
import { MockNgControl } from '../../../test/form/MockNgControl';

describe('FormControlEditorAbstractComponent', () => {
  let component: TestComponent;
  let fixture: ComponentFixture<TestComponent>;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [ReactiveFormsModule, TestComponent],
      providers: [FormGroupDirective],
    });

    fixture = TestBed.createComponent(TestComponent);
    component = fixture.componentInstance;
    component.control = new MockNgControl();
    fixture.detectChanges();
  });

  describe('ngOnInit', () => {
    it('should set subscription', () => {
      component.ngOnInit();

      expect(component._changesSubscription).toBeDefined();
    });

    it('should handle field control value changes ', () => {
      component.handleFieldControlValueChange = jest.fn();
      component.ngOnInit();

      component.fieldControl.setValue('testValue');

      expect(component.handleFieldControlValueChange).toHaveBeenCalledWith('testValue');
    });

    it('should set subscription', () => {
      component.ngOnInit();

      expect(component._statusSubscription).toBeDefined();
    });

    it('should set errors on statusChange', () => {
      component.setErrors = jest.fn();
      component.ngOnInit();

      component.control.control.setErrors({ required: true });

      expect(component.setErrors).toHaveBeenCalled();
    });
  });

  describe('writeValue', () => {
    it('should set fieldControl value', () => {
      const value = 'testValue';

      component.writeValue(value);

      expect(component.fieldControl.value).toBe(value);
    });
  });

  describe('setErrors', () => {
    it('should set fieldControl errors', () => {
      const errors: ValidationErrors = { required: true };

      component.control.control.setErrors(errors);

      expect(component.fieldControl.errors).toEqual(errors);
    });

    it('should set fieldControl to touched', () => {
      component.fieldControl.markAsPristine();

      component.setErrors();

      expect(component.fieldControl.touched).toBe(true);
    });

    it('should update invalid params', () => {
      component._updateInvalidParams = jest.fn();

      component.setErrors();

      expect(component._updateInvalidParams).toHaveBeenCalled();
    });
  });

  describe('removeErrors', () => {
    it('should remove fieldControl errors', () => {
      component.fieldControl.setErrors({ fehler: 'this is an validation error' });

      component._removeErrors();

      expect(component.fieldControl.errors).toBeNull();
    });

    it('should update invalid params', () => {
      component._updateInvalidParams = jest.fn();

      component._removeErrors();

      expect(component._updateInvalidParams).toHaveBeenCalled();
    });
  });
});

@Component({
  standalone: true,
  template: '',
  imports: [CommonModule, ReactiveFormsModule],
  providers: [FormGroupDirective],
})
class TestComponent extends FormControlEditorAbstractComponent {}
