/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { argsToTemplate, moduleMetadata, type Meta, type StoryObj } from '@storybook/angular';

import { AttachmentComponent } from '../attachment/attachment.component';
import { IconComponent } from '../icon/icon.component';
import { AttachmentHeaderComponent } from './attachment-header.component';

const meta: Meta<AttachmentHeaderComponent> = {
  title: 'Attachment header',
  component: AttachmentHeaderComponent,
  parameters: {
    docs: {
      description: {
        component: 'Header may contain title and/or buttons',
      },
    },
  },
  decorators: [
    moduleMetadata({
      imports: [AttachmentHeaderComponent, AttachmentComponent, IconComponent],
    }),
  ],
  excludeStories: /.*Data$/,
  tags: ['autodocs'],
};

export default meta;
type Story = StoryObj<AttachmentHeaderComponent>;

export const Default: Story = {
  args: {
    title: 'Anhänge',
  },
  argTypes: {
    title: {
      description: 'Title for group of files',
    },
  },
  render: (args) => ({
    props: args,
    template: `<ods-attachment-header ${argsToTemplate(args)}>
        <button action-buttons>
          <ods-icon name="save" size="small" fill="text" />
        </button>
    </ods-attachment-header>`,
  }),
};
