/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { existsAsHtmlElement, getElementFromFixture, notExistsAsHtmlElement } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { IconComponent } from '../../icon/icon.component';
import { ButtonToggleComponent } from './button-toggle.component';

describe('ButtonToggleComponent', () => {
  let component: ButtonToggleComponent;
  let fixture: ComponentFixture<ButtonToggleComponent>;
  const checkIconSelector = getDataTestIdOf('toggle-check-icon');
  const buttonToggleSelector = getDataTestIdOf('button-toggle');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      imports: [ButtonToggleComponent, MockComponent(IconComponent)],
    }).compileComponents();

    fixture = TestBed.createComponent(ButtonToggleComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('component', () => {
    describe('isFocused', () => {
      it('should be set to true on focus', () => {
        const buttonToggle: HTMLButtonElement = getElementFromFixture(fixture, buttonToggleSelector);

        buttonToggle.focus();
        fixture.detectChanges();

        expect(component.isFocused).toBe(true);
      });

      it('should be set to false on blue', () => {
        component.isFocused = true;
        const buttonToggle: HTMLButtonElement = getElementFromFixture(fixture, buttonToggleSelector);

        buttonToggle.dispatchEvent(new Event('blur'));
        fixture.detectChanges();

        expect(component.isFocused).toBe(false);
      });
    });

    describe('ngDoCheck', () => {
      it('should call isSelected', () => {
        component.isSelected = jest.fn();
        component.value = 'test';

        component.ngDoCheck();

        expect(component.isSelected).toHaveBeenCalledWith('test');
      });

      it('should set isChecked', () => {
        component.isSelected = jest.fn().mockReturnValue(true);

        component.ngDoCheck();

        expect(component.isChecked).toBe(true);
      });
    });

    describe('isToggleChecked', () => {
      it('should return isChecked', () => {
        component.isChecked = true;

        expect(component.isToggleChecked).toBe(true);
      });
    });
  });

  describe('template', () => {
    describe('check icon', () => {
      it('should show', () => {
        component.isSelected = jest.fn().mockReturnValue(true);

        fixture.detectChanges();

        existsAsHtmlElement(fixture, checkIconSelector);
      });

      it('should hide', () => {
        component.isSelected = jest.fn().mockReturnValue(false);

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, checkIconSelector);
      });
    });
  });
});
