/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { CommonModule } from '@angular/common';
import { argsToTemplate, moduleMetadata, type Meta, type StoryObj } from '@storybook/angular';

import { IconComponent } from '../../icon/icon.component';
import { SpinnerIconComponent } from '../../icons/spinner-icon/spinner-icon.component';
import { FileUploadButtonComponent } from './file-upload-button.component';

const meta: Meta<FileUploadButtonComponent> = {
  title: 'File upload button',
  component: FileUploadButtonComponent,
  decorators: [
    moduleMetadata({
      imports: [CommonModule, FileUploadButtonComponent, SpinnerIconComponent, IconComponent],
    }),
  ],
  excludeStories: /.*Data$/,
  tags: ['autodocs'],
};

export default meta;
type Story = StoryObj<FileUploadButtonComponent>;

export const Upload: Story = {
  args: {
    id: '1',
    isLoading: false,
  },
  render: (args: FileUploadButtonComponent) => ({
    props: args,
    template: `<ods-file-upload-button ${argsToTemplate(args)}>
      <ods-icon icon name="attachment" />
      <ods-spinner-icon spinner size="medium"></ods-spinner-icon>
      <p text>Anhang hochladen</p>
    </ods-file-upload-button>`,
  }),
};
