/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { CommonModule } from '@angular/common';
import { FormGroupDirective } from '@angular/forms';
import { argsToTemplate, moduleMetadata, type Meta, type StoryObj } from '@storybook/angular';

import { IconComponent } from '../../icon/icon.component';
import { RadioButtonCardComponent } from './radio-button-card.component';

const meta: Meta<RadioButtonCardComponent> = {
  title: 'Radio button card',
  component: RadioButtonCardComponent,
  decorators: [
    moduleMetadata({
      imports: [CommonModule, RadioButtonCardComponent, IconComponent],
      providers: [FormGroupDirective],
    }),
  ],
  excludeStories: /.*Data$/,
  tags: ['autodocs'],
};

export default meta;
type Story = StoryObj<RadioButtonCardComponent>;

export const Default: Story = {
  args: {
    label: 'bewilligt',
    name: 'exampleName',
    value: 'bewilligt',
    variant: 'bescheid_bewilligt',
    fullWidthText: false,
  },
  render: (args: RadioButtonCardComponent) => ({
    props: args,
    template: `<div class="my-10 flex gap-8">
    <ods-radio-button-card
      ${argsToTemplate(args)}
    >
      <ods-icon name="stamp" fill="bewilligt" />
    </ods-radio-button-card>
    <ods-radio-button-card
      label="abgelehnt"
      name="exampleName"
      value="abgelehnt"
      variant="bescheid_abgelehnt"
    >
      <ods-icon name="close" fill="abgelehnt" size="large" />
    </ods-radio-button-card>
  </div>`,
  }),
};

export const Message: Story = {
  args: {
    label: 'bewilligt',
    name: 'exampleName',
    value: 'bewilligt',
    variant: 'bescheid_bewilligt',
    fullWidthText: false,
  },
  render: (args: RadioButtonCardComponent) => ({
    props: args,
    template: `<div
    class="flex w-full max-w-72 flex-col gap-4"
  >
      <ods-radio-button-card
        label="Als neue Nachricht an den Antragsteller senden"
        value="NACHRICHT"
        name="nachrichtOrManual"
        variant="bescheid_save"
        [fullWidthText]="true"
      >
        <ods-icon name="send" />
      </ods-radio-button-card>
      <ods-radio-button-card
        label="Nur speichern"
        value="MANUAL"
        name="nachrichtOrManual"
        variant="bescheid_save"
        [fullWidthText]="true"
      >
        <ods-icon name="save" />
      </ods-radio-button-card>
  </div>`,
  }),
};
