/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { moduleMetadata, type Meta, type StoryObj } from '@storybook/angular';

import { CommonModule } from '@angular/common';
import { IconComponent } from './icon.component';

const meta: Meta<IconComponent> = {
  title: 'Icon',
  parameters: {
    docs: {
      description: {
        component: 'Generic icon component',
      },
    },
  },
  component: IconComponent,
  decorators: [
    moduleMetadata({
      imports: [IconComponent, CommonModule],
    }),
  ],
  excludeStories: /.*Data$/,
  tags: ['autodocs'],
};

const ICONS_LIST: string[] = [
  'accessibility',
  'account-circle',
  'admin-logo',
  'archive',
  'arrow-back',
  'attachment',
  'bar-chart-arrow',
  'bescheid-generate',
  'bescheid-upload',
  'bubble-question-mark',
  'check-circle',
  'check',
  'close',
  'delete-vorgang-finally',
  'delete',
  'discard-vorgang',
  'edit',
  'error',
  'exclamation',
  'external-unit',
  'forward-vorgang',
  'help',
  'info',
  'logout',
  'mail',
  'more',
  'office',
  'open-link',
  'orga-unit',
  'ozg-logo',
  'person',
  'plus',
  'public-administration',
  'request-vorgang-deletion',
  'save',
  'search',
  'send',
  'settings',
  'stamp',
  'statistic',
  'undo-request-vorgang-deletion',
  'undo',
  'update',
  'user',
  'users',
];
const stringifiedIcons: string = JSON.stringify(ICONS_LIST);

export default meta;
type Story = StoryObj<IconComponent>;

export const Default: Story = {
  args: {
    name: 'mail',
    size: 'extra-large',
    fill: 'primary',
    class: '',
  },
  argTypes: {
    name: { description: 'Name of the icon' },
    size: {
      description: 'Size of the icon. "Full" stands for 100%',
      control: 'select',
      options: ['small', 'medium', 'large', 'extra-large', 'xxl', 'full'],
      table: { defaultValue: { summary: 'medium' } },
    },
    fill: {
      description: 'Fill color of the icon',
      control: 'select',
      options: ['primary', 'text', 'whitetext', 'neutral', 'error', 'pdf', 'bewilligt', 'abgelehnt'],
      table: { defaultValue: { summary: 'primary' } },
    },
    class: { description: 'Additional (tailwind) classes for the icon' },
  },
};

export const AllIcons: Story = {
  render: () => {
    return {
      template: `<div class="flex gap-6 w-full flex-wrap justify-around">
        @for (icon of ${stringifiedIcons}; track $index) {
          <div class="flex flex-col items-center">
            <ods-icon [name]="icon" size="extra-large" />
            <p class="text-pretty">{{ icon }}</p>
          </div>
        }
      </div>`,
    };
  },
};
