/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { CommandResource } from '@alfa-client/command-shared';
import { StateResource } from '@alfa-client/tech-shared';
import { VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { Component, EventEmitter, Input, OnChanges, Output, SimpleChanges } from '@angular/core';
import { UntypedFormGroup } from '@angular/forms';
import { isNil } from 'lodash-es';
import { ForwardingByEmailFormService } from './forwarding-by-email-form.service';

@Component({
  selector: 'alfa-forward-by-email-form',
  templateUrl: './forwarding-by-email-form.component.html',
  styleUrls: ['./forwarding-by-email-form.component.scss'],
})
export class ForwardingByEmailFormComponent implements OnChanges {
  @Input() forwardStateResource: StateResource<CommandResource>;
  @Input() vorgang: VorgangWithEingangResource;

  @Output('doSubmit') submit: EventEmitter<void> = new EventEmitter();

  readonly formServiceClass = ForwardingByEmailFormService;

  constructor(private formService: ForwardingByEmailFormService) {}

  ngOnChanges(changes: SimpleChanges) {
    if (changes.vorgang && this.zustaendigeStelleEmailExists()) {
      this.patchEmail();
    }
  }

  zustaendigeStelleEmailExists(): boolean {
    return (
      this.vorgang &&
      this.vorgang.eingang &&
      this.vorgang.eingang.zustaendigeStelle &&
      !isNil(this.vorgang.eingang.zustaendigeStelle.email)
    );
  }

  patchEmail(): void {
    this.formService.patchField(ForwardingByEmailFormService.FIELD_EMAIL, this.vorgang.eingang.zustaendigeStelle.email);
  }

  get form(): UntypedFormGroup {
    return this.formService.form;
  }
}
