import { EMPTY_STRING } from '@alfa-client/tech-shared';
import { dispatchEventFromFixture, MockEvent } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ButtonWithSpinnerComponent } from '@ods/component';
import { IconComponent } from '@ods/system';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { ForwardingButtonComponent } from './forwarding-button.component';

describe('ForwardingButtonComponent', () => {
  let component: ForwardingButtonComponent;
  let fixture: ComponentFixture<ForwardingButtonComponent>;

  const button: string = getDataTestIdOf('forwarding-button-container');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      imports: [ForwardingButtonComponent],
      declarations: [MockComponent(IconComponent), MockComponent(ButtonWithSpinnerComponent)],
    }).compileComponents();

    fixture = TestBed.createComponent(ForwardingButtonComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('component', () => {
    describe('set disabled', () => {
      it('should set tooltip text', () => {
        component.disabled = true;

        expect(component.tooltip).not.toBe(EMPTY_STRING);
      });

      it('should set empty tooltip', () => {
        component.disabled = false;

        expect(component.tooltip).toBe(EMPTY_STRING);
      });
    });
  });

  describe('template', () => {
    describe('on button click', () => {
      it('should emit', () => {
        component.clickEmitter.emit = jest.fn();

        dispatchEventFromFixture(fixture, button, MockEvent.CLICK);

        expect(component.clickEmitter.emit).toHaveBeenCalled();
      });
    });
  });
});
