import {
  CommandListResourceService,
  CommandListResourceServiceConfig,
  CommandOrder,
  CommandService,
} from '@alfa-client/command-shared';
import { ResourceRepository } from '@alfa-client/tech-shared';
import { VorgangHeaderLinkRel, VorgangService, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { KommentarListLinkRel } from './kommentar.linkrel';
import { KommentarListResource, KommentarResource } from './kommentar.model';
import { KOMMENTAR_FEATURE_KEY, KOMMENTAR_LIST_PATH } from './kommentar.state';

export class KommentarListResourceService extends CommandListResourceService<
  VorgangWithEingangResource,
  KommentarListResource,
  KommentarResource
> {}

export function createKommentarListResourceService(
  vorgangService: VorgangService,
  repository: ResourceRepository<KommentarResource>,
  commandService: CommandService,
): KommentarListResourceService {
  return new KommentarListResourceService(buildConfig(vorgangService), repository, commandService);
}

export function buildConfig(vorgangService: VorgangService): CommandListResourceServiceConfig<VorgangWithEingangResource> {
  return {
    stateInfo: { name: KOMMENTAR_FEATURE_KEY, path: KOMMENTAR_LIST_PATH },
    baseResource: vorgangService.selectVorgangWithEingang(),
    getLinkRel: VorgangHeaderLinkRel.KOMMENTARE,
    listResourceListLinkRel: KommentarListLinkRel.KOMMENTAR_LIST,
    create: {
      linkRel: KommentarListLinkRel.CREATE_KOMMENTAR,
      order: CommandOrder.CREATE_KOMMENTAR,
    },
  };
}
