/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { KommentarListLinkRel, KommentarService } from '@alfa-client/kommentar-shared';
import { HasLinkPipe, createStateResource } from '@alfa-client/tech-shared';
import { dispatchEventFromFixture, existsAsHtmlElement, mock, notExistsAsHtmlElement } from '@alfa-client/test-utils';
import { ExpansionPanelComponent, OzgcloudStrokedButtonWithSpinnerComponent } from '@alfa-client/ui';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { SpinnerComponent } from '@ods/component';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { createKommentarListResource } from '../../../../kommentar-shared/test/kommentar';
import { getDataTestIdAttributeOf, getDataTestIdOf } from '../../../../tech-shared/test/data-test';
import '../../../../tech-shared/test/marbles';
import { createVorgangWithEingangResource } from '../../../../vorgang-shared/test/vorgang';
import { KommentarFormComponent } from './kommentar-form/kommentar-form.component';
import { KommentarListInVorgangContainerComponent } from './kommentar-list-in-vorgang-container.component';
import { KommentarListInVorgangComponent } from './kommentar-list-in-vorgang/kommentar-list-in-vorgang.component';

describe('KommentarListInVorgangContainerComponent', () => {
  let component: KommentarListInVorgangContainerComponent;
  let fixture: ComponentFixture<KommentarListInVorgangContainerComponent>;

  const createCommentButton = getDataTestIdAttributeOf('create-kommentar');
  const commentForm = getDataTestIdOf('kommentar-formular');

  const service = mock(KommentarService);
  service.getKommentare.mockReturnValue(
    of(createStateResource(createKommentarListResource([KommentarListLinkRel.CREATE_KOMMENTAR]))),
  );

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        KommentarListInVorgangContainerComponent,
        MockComponent(ExpansionPanelComponent),
        MockComponent(KommentarListInVorgangComponent),
        MockComponent(KommentarFormComponent),
        MockComponent(OzgcloudStrokedButtonWithSpinnerComponent),
        MockComponent(SpinnerComponent),
        HasLinkPipe,
      ],
      providers: [
        {
          provide: KommentarService,
          useValue: service,
        },
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(KommentarListInVorgangContainerComponent);
    component = fixture.componentInstance;
    component.vorgangStateResource = createStateResource(createVorgangWithEingangResource());
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('template', () => {
    describe('alfa-kommentar-form', () => {
      it('should be visible when showFormular is true', () => {
        component.showFormular$ = of(true);
        fixture.detectChanges();

        existsAsHtmlElement(fixture, commentForm);
      });

      it('should not be visible when showFormular is false', () => {
        component.showFormular$ = of(false);
        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, commentForm);
      });

      it('should call hideFormular when clicked', () => {
        component.showFormular$ = of(true);
        component.hideFormular = jest.fn();
        fixture.detectChanges();

        dispatchEventFromFixture(fixture, commentForm, 'cancel');

        expect(component.hideFormular).toHaveBeenCalled();
      });
    });

    describe('ozgcloud-stroked-button-with-spinner', () => {
      it('should be visible when Formular is not shown', () => {
        component.showFormular$ = of(false);
        fixture.detectChanges();

        existsAsHtmlElement(fixture, createCommentButton);
      });

      it('should not be visible when Formular is shown', () => {
        component.showFormular$ = of(true);
        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, createCommentButton);
      });

      it('should call show Formular when clicked', () => {
        component.showFormular = jest.fn();

        dispatchEventFromFixture(fixture, createCommentButton, 'clickEmitter');

        expect(component.showFormular).toHaveBeenCalled();
      });
    });
  });

  describe('component', () => {
    describe('ng onInit', () => {
      beforeEach(() => {
        service.isNewKommentarFormularVisible.mockReturnValue(of(false));
      });

      it('should call kommentar service isFormularVisible', () => {
        component.ngOnInit();

        expect(service.isNewKommentarFormularVisible).toHaveBeenCalled();
      });

      it('should call kommentar service getKommentare', () => {
        component.ngOnInit();

        expect(service.getKommentare).toHaveBeenCalled();
      });
    });

    describe('showFormular', () => {
      it('should call kommentar service showFormular', () => {
        component.showFormular();

        expect(service.showNewKommentarFormular).toHaveBeenCalled();
      });
    });

    describe('hide formular', () => {
      it('should call kommentar service hideFormular', () => {
        component.hideFormular();

        expect(service.hideNewKommentarFormular).toHaveBeenCalled();
      });
    });
  });
});
