/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BinaryFileListResource } from '@alfa-client/binary-file-shared';
import { KommentarLinkRel, KommentarListResource, KommentarResource, KommentarService } from '@alfa-client/kommentar-shared';
import { createEmptyStateResource, StateResource } from '@alfa-client/tech-shared';
import { Component, inject, Input, OnInit } from '@angular/core';
import { getUrl, hasLink, ResourceUri } from '@ngxp/rest';
import { Observable, of } from 'rxjs';

@Component({
  selector: 'alfa-kommentar-list-item-in-vorgang',
  templateUrl: './kommentar-list-item-in-vorgang.component.html',
  styleUrls: ['./kommentar-list-item-in-vorgang.component.scss'],
})
export class KommentarListItemInVorgangComponent implements OnInit {
  public kommentarService = inject(KommentarService);

  @Input() public kommentar: KommentarResource;
  @Input() public kommentarListStateResource: StateResource<KommentarListResource>;

  @Input() set currentlyEdited(value: ResourceUri) {
    this.editMode &&= value === getUrl(this.kommentar);
  }

  public editMode: boolean = false;
  public kommentarStateResource$: Observable<StateResource<KommentarResource>>;
  public attachments$: Observable<StateResource<BinaryFileListResource>> = of(createEmptyStateResource<BinaryFileListResource>());

  public readonly KommentarLinkRel = KommentarLinkRel;

  ngOnInit(): void {
    this.attachments$ = this.kommentarService._getAttachments(this.kommentar);
  }

  edit(): void {
    this.kommentarStateResource$ = this.kommentarService.getKommentarStateResource();
    this.kommentarService.selectKommentar(this.kommentar);
    this.kommentarService.setCurrentlyEdited(getUrl(this.kommentar));
    this.editMode = hasLink(this.kommentar, KommentarLinkRel.EDIT);
  }
}
