/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { mock, useFromMock } from '@alfa-client/test-utils';
import { VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { ResourceFactory } from '@ngxp/rest';
import { cold, hot } from 'jest-marbles';
import { createLoeschAnforderungResource } from 'libs/loesch-anforderung-shared/test/loesch-anforderung';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { LoeschAnforderungResource } from '../loesch-anforderung.model';
import { LoeschAnforderungRepository } from './loesch-anforderung.repository';

describe('LoeschAnforderungRepository', () => {
  let repository: LoeschAnforderungRepository;
  let resourceFactory = mock(ResourceFactory);
  let resourceWrapper = { get: jest.fn() };

  beforeEach(() => {
    repository = new LoeschAnforderungRepository(useFromMock(resourceFactory));

    resourceFactory.fromId.mockReturnValue(resourceWrapper);
    resourceFactory.from.mockReturnValue(resourceWrapper);
  });

  describe('getLoeschAnforderung', () => {
    const vorgangWithEingangResource: VorgangWithEingangResource =
      createVorgangWithEingangResource();
    const loeschAnforderungResource: LoeschAnforderungResource = createLoeschAnforderungResource();

    beforeEach(() => {
      resourceWrapper.get.mockReturnValue(hot('a', { a: loeschAnforderungResource }));
    });

    it('should call resourceFactory with resource', () => {
      repository.getLoeschAnforderung(vorgangWithEingangResource);

      expect(resourceFactory.from).toHaveBeenCalledWith(vorgangWithEingangResource);
    });

    it('should call resourceWrapper with link', () => {
      repository.getLoeschAnforderung(vorgangWithEingangResource);

      expect(resourceWrapper.get).toHaveBeenCalledWith(
        VorgangWithEingangLinkRel.LOESCH_ANFORDERUNG,
      );
    });

    it('should return result', () => {
      let result = repository.getLoeschAnforderung(vorgangWithEingangResource);

      expect(result).not.toBeNull();
      expect(result).toBeObservable(cold('a', { a: loeschAnforderungResource }));
    });
  });
});
