/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { LoeschAnforderungResource, LoeschAnforderungService } from '@alfa-client/loesch-anforderung-shared';
import { HasLinkPipe, StateResource, createEmptyStateResource, createStateResource } from '@alfa-client/tech-shared';
import { mock } from '@alfa-client/test-utils';
import { BasicDialogComponent, BasicDialogData, DialogService } from '@alfa-client/ui';
import { VorgangService } from '@alfa-client/vorgang-shared';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatDialogModule, MatDialogRef } from '@angular/material/dialog';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { createCommandResource } from 'libs/command-shared/test/command';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { EndgueltigLoeschenButtonContainerComponent } from './endgueltig-loeschen-button-container.component';
import { EndgueltigLoeschenButtonComponent } from './endgueltig-loeschen-button/endgueltig-loeschen-button.component';

describe('EndgueltigLoeschenButtonContainerComponent', () => {
  let component: EndgueltigLoeschenButtonContainerComponent;
  let fixture: ComponentFixture<EndgueltigLoeschenButtonContainerComponent>;

  const loeschAnforderungResource: StateResource<LoeschAnforderungResource> = createEmptyStateResource();

  const dialogRef = <MatDialogRef<BasicDialogComponent>>{};
  dialogRef.afterClosed = jest.fn().mockReturnValue(of(true));

  const loeschAnforderungService = mock(LoeschAnforderungService);
  const vorgangService = mock(VorgangService);
  const dialogService = mock(DialogService);

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [EndgueltigLoeschenButtonContainerComponent, HasLinkPipe, MockComponent(EndgueltigLoeschenButtonComponent)],
      imports: [BrowserAnimationsModule, MatDialogModule],
      providers: [
        {
          provide: MatDialogRef,
          useValue: {},
        },
        {
          provide: LoeschAnforderungService,
          useValue: loeschAnforderungService,
        },
        {
          provide: VorgangService,
          useValue: vorgangService,
        },
        {
          provide: DialogService,
          useValue: dialogService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(EndgueltigLoeschenButtonContainerComponent);
    component = fixture.componentInstance;
    loeschAnforderungService.getEndgueltigLoeschenCommand.mockReturnValue(of(createStateResource(createCommandResource())));
    dialogService.openBasic = jest.fn().mockReturnValue(dialogRef);
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    it('should call vorgangCommandService#getEndgueltigLoeschenCommand', () => {
      component.ngOnInit();

      expect(loeschAnforderungService.getEndgueltigLoeschenCommand).toHaveBeenCalled();
    });

    it('should call loeschAnforderungService#getLoeschAnforderung', () => {
      component.ngOnInit();

      expect(loeschAnforderungService.getLoeschAnforderung).toHaveBeenCalled();
    });
  });

  describe('openEndgueltigLoeschenDialog', () => {
    it('should call dialogService.openBasic', () => {
      component.openEndgueltigLoeschenDialog(loeschAnforderungResource);

      expect(dialogService.openBasic).toHaveBeenCalled();
    });

    it('should call createBasicDialogConfig', () => {
      component.createBasicDialogConfig = jest.fn();

      component.openEndgueltigLoeschenDialog(loeschAnforderungResource);

      expect(component.createBasicDialogConfig).toHaveBeenCalled();
    });

    it('should call dialogRef.afterClosed', () => {
      component.openEndgueltigLoeschenDialog(loeschAnforderungResource);

      expect(dialogRef.afterClosed).toHaveBeenCalled();
    });

    it('should call component.endgueltigLoeschen', () => {
      component.endgueltigLoeschen = jest.fn();

      component.openEndgueltigLoeschenDialog(loeschAnforderungResource);

      expect(component.endgueltigLoeschen).toHaveBeenCalled();
    });
  });

  describe('createBasicDialogConfig', () => {
    it('should return BasicDialogData', () => {
      const basicDialogData: BasicDialogData = {
        title: 'Möchten Sie den Vorgang endgültig löschen?',
        message: 'Diese Aktion kann nicht widerrufen werden.',
        okText: 'Endgültig löschen',
        okSvgIcon: 'delete-vorgang-finally',
      };

      const res = component.createBasicDialogConfig();

      expect(res).toMatchObject<BasicDialogData>(basicDialogData);
    });
  });

  describe('endgueltigLoeschen', () => {
    it('should call service', () => {
      const spy = jest.spyOn(loeschAnforderungService, 'endgueltigLoeschen');

      component.endgueltigLoeschen(loeschAnforderungResource);

      expect(spy).toHaveBeenCalledWith(loeschAnforderungResource.resource);
    });
  });
});
