/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { CommandResource } from '@alfa-client/command-shared';
import {
  LoeschAnforderungLinkRel,
  LoeschAnforderungResource,
  LoeschAnforderungService,
} from '@alfa-client/loesch-anforderung-shared';
import { StateResource, createEmptyStateResource } from '@alfa-client/tech-shared';
import { Component, Input } from '@angular/core';
import { Observable, of } from 'rxjs';

@Component({
  selector: 'alfa-loesch-anforderung-zuruecknehmen-button-container',
  templateUrl: './loesch-anforderung-zuruecknehmen-button-container.component.html',
  styles: [':host {@apply empty:hidden}'],
})
export class LoeschAnforderungZuruecknehmenButtonContainerComponent {
  @Input() showAsIconButton: boolean = false;

  loeschAnforderungStateResource$: Observable<StateResource<LoeschAnforderungResource>>;
  loeschAnforderungZuruecknehmenCommand$: Observable<StateResource<CommandResource>> =
    of(createEmptyStateResource<CommandResource>());

  readonly loeschAnforderungLinkRel = LoeschAnforderungLinkRel;

  constructor(private loeschAnforderungService: LoeschAnforderungService) {}

  ngOnInit(): void {
    this.loeschAnforderungZuruecknehmenCommand$ = this.loeschAnforderungService.getLoschAnforderungZuruecknehmenCommand();
    this.loeschAnforderungStateResource$ = this.loeschAnforderungService.getLoeschAnforderung();
  }

  public loeschAnforderungZuruecknehmen(loeschAnforderungResource: StateResource<LoeschAnforderungResource>): void {
    this.loeschAnforderungService.zuruecknehmen(loeschAnforderungResource.resource);
  }
}
