/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { Router, UrlTree } from '@angular/router';
import { RouterTestingModule } from '@angular/router/testing';
import { faker } from '@faker-js/faker';
import { OzgCloudUrlSerializer } from './ozgcloud-url-serializer';

import { TestBed } from '@angular/core/testing';
import * as NavigationUtil from './navigation.util';

describe('OzgCloudUrlSerializer', () => {
  let serializer: OzgCloudUrlSerializer;
  let router: Router;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [RouterTestingModule],
    });

    router = TestBed.inject(Router);
    serializer = new OzgCloudUrlSerializer();
  });

  it('should be created', () => {
    expect(serializer).toBeTruthy();
  });

  describe('parse', () => {
    it('should return URLTree', () => {
      const url: string = faker.internet.url();

      const urlTree: UrlTree = serializer.parse(url);

      expect(urlTree).toBeInstanceOf(UrlTree);
    });
  });

  describe('serialize', () => {
    it('should call revertDoubleEncodingOfPercentageSign', () => {
      const spy: jest.SpyInstance<string, [text: string]> = jest.spyOn(
        NavigationUtil,
        'revertDoubleEncodingOfPercentageSign',
      );
      const urlTree: UrlTree = router.createUrlTree(['/']);

      serializer.serialize(urlTree);

      expect(spy).toHaveBeenCalled();
    });

    it('should ensure percentage sign is not double encoded', () => {
      const urlTree: UrlTree = router.createUrlTree(['/', '%25']);

      const path: string = serializer.serialize(urlTree);

      expect(path).toBe('/%25');
    });
  });
});
