/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { OrganisationsEinheitResourceService } from '@alfa-client/organisations-einheit-shared';
import { createStateResource, StateResource } from '@alfa-client/tech-shared';
import { mock, Mock } from '@alfa-client/test-utils';
import { OrganisationsEinheitResource } from '@alfa-client/zustaendige-stelle-shared';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { toResource } from 'libs/tech-shared/test/resource';
import { createOrganisationsEinheit } from 'libs/zustaendige-stelle-shared/test/organisations-einheit';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { OrganisationsEinheitListItemContainerComponent } from './organisations-einheit-list-item-container.component';
import { OrganisationsEinheitListItemComponent } from './organisations-einheit-list-item/organisations-einheit-list-item.component';

describe('OrganisationsEinheitListItemContainerComponent', () => {
  let component: OrganisationsEinheitListItemContainerComponent;
  let fixture: ComponentFixture<OrganisationsEinheitListItemContainerComponent>;

  let organisationsEinheitResourceService: Mock<OrganisationsEinheitResourceService>;

  const organisationsEinheit: OrganisationsEinheitResource = toResource(createOrganisationsEinheit());
  const organisationsEInheitStateResource: StateResource<OrganisationsEinheitResource> =
    createStateResource(organisationsEinheit);

  beforeEach(async () => {
    organisationsEinheitResourceService = {
      ...mock(OrganisationsEinheitResourceService),
      getOrganisationsEinheit: jest.fn().mockReturnValue(of(organisationsEInheitStateResource)),
    };

    await TestBed.configureTestingModule({
      imports: [OrganisationsEinheitListItemContainerComponent, MockComponent(OrganisationsEinheitListItemComponent)],
    })
      .overrideComponent(OrganisationsEinheitListItemContainerComponent, {
        set: {
          providers: [{ provide: OrganisationsEinheitResourceService, useValue: organisationsEinheitResourceService }],
        },
      })
      .compileComponents();

    fixture = TestBed.createComponent(OrganisationsEinheitListItemContainerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('component', () => {
    describe('ngOnInit', () => {
      it('should call organisationsEinheitResourceService getOrganisationsEinheit Resource', () => {
        component.ngOnInit();

        expect(organisationsEinheitResourceService.getOrganisationsEinheit).toHaveBeenCalled();
      });
    });
  });
});
