/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { PostfachMailListResource } from '@alfa-client/postfach-shared';
import {
  ConvertForDataTestPipe,
  createEmptyStateResource,
  createStateResource,
  HasLinkPipe,
  ToEmbeddedResourcesPipe,
} from '@alfa-client/tech-shared';
import { existsAsHtmlElement, notExistsAsHtmlElement } from '@alfa-client/test-utils';
import { SpinnerComponent } from '@alfa-client/ui';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MockComponent } from 'ng-mocks';
import { createPostfachMailListResource } from '../../../../../../postfach-shared/test/postfach';
import { getDataTestIdOf } from '../../../../../../tech-shared/test/data-test';
import { PostfachMailComponent } from '../../../postfach-mail-list-container/postfach-mail-list/postfach-mail/postfach-mail.component';
import { PostfachMailPdfButtonContainerComponent } from '../../../postfach-mail-pdf-button-container/postfach-mail-pdf-button-container.component';
import { MailUnreadButtonLinkContainerComponent } from '../mail-unread-button/mail-unread-button-link-container.component';
import { PostfachPageMailListComponent } from './postfach-page-mail-list.component';

describe('PostfachPageMailListComponent', () => {
  let component: PostfachPageMailListComponent;
  let fixture: ComponentFixture<PostfachPageMailListComponent>;

  const mailList: string = getDataTestIdOf('postfach-page-mail-list');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        PostfachPageMailListComponent,
        ToEmbeddedResourcesPipe,
        ConvertForDataTestPipe,
        HasLinkPipe,
        MockComponent(PostfachMailComponent),
        MockComponent(SpinnerComponent),
        MockComponent(PostfachMailPdfButtonContainerComponent),
        MockComponent(MailUnreadButtonLinkContainerComponent),
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(PostfachPageMailListComponent);
    component = fixture.componentInstance;
    component.postfachMailListStateResource = createStateResource<PostfachMailListResource>(createPostfachMailListResource());
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('mail list', () => {
    it('should exist if resource', () => {
      component.postfachMailListStateResource = createStateResource<PostfachMailListResource>(createPostfachMailListResource());

      fixture.detectChanges();

      existsAsHtmlElement(fixture, mailList);
    });

    it('should not exist if no resource', () => {
      component.postfachMailListStateResource = createEmptyStateResource<PostfachMailListResource>();

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, mailList);
    });
  });
});
