import { Actions, createEffect, ofType } from '@ngrx/effects';
import { Resource } from '@ngxp/rest';
import { of } from 'rxjs';
import { catchError, map, switchMap } from 'rxjs/operators';
import { ResourceRepository } from '../resource/resource.repository';
import { ResourceUriProps, SingleResourceLoadActions } from './actions';

export class ResourceEffects<T extends Resource> {
  constructor(
    private actions$: Actions,
    private repository: ResourceRepository,
    private resourceActions: SingleResourceLoadActions,
  ) {}

  loadByUri$ = createEffect(() =>
    this.actions$.pipe(
      ofType(this.resourceActions.loadAction),
      switchMap((props: ResourceUriProps) => {
        return this.repository.getResource<T>(props.resourceUri).pipe(
          map((resource: T) => this.resourceActions.loadSuccessAction({ resource })),
          catchError((error) => of(this.resourceActions.loadFailureAction({ error }))),
        );
      }),
    ),
  );
}
