import { Action, ActionReducer, createReducer, on } from '@ngrx/store';
import {
  createEmptyStateResource,
  createErrorStateResource,
  createStateResource,
} from '../resource/resource.util';
import {
  LoadResourceFailureProps,
  LoadResourceSuccessProps,
  SingleResourceLoadActions,
} from './actions';
import { SingleResourceState, initialResourceState } from './state.model';

export class SingleResourceReducer {
  public reducer: ActionReducer<SingleResourceState, Action>;

  constructor(private actions: SingleResourceLoadActions) {
    this.initReducer();
  }

  initReducer(): void {
    this.reducer = createReducer(
      initialResourceState,
      on(
        this.actions.loadAction,
        (state: SingleResourceState): SingleResourceState => ({
          ...state,
          resource: { ...state.resource, loading: true },
        }),
      ),
      on(
        this.actions.loadSuccessAction,
        (state: SingleResourceState, props: LoadResourceSuccessProps): SingleResourceState => ({
          ...state,
          resource: createStateResource(props.resource),
        }),
      ),
      on(
        this.actions.loadFailureAction,
        (state: SingleResourceState, props: LoadResourceFailureProps): SingleResourceState => ({
          ...state,
          resource: createErrorStateResource(props.error),
        }),
      ),
      on(
        this.actions.clearAction,
        (state: SingleResourceState): SingleResourceState => ({
          ...state,
          resource: createEmptyStateResource(),
        }),
      ),
      on(
        this.actions.reloadAction,
        (state: SingleResourceState): SingleResourceState => ({
          ...state,
          resource: { ...state.resource, reload: true },
        }),
      ),
    );
  }
}
