/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { FormatToPrettyDatePipe } from './format-to-pretty-date.pipe';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('FormatToPrettyDatePipe', () => {
  let pipe: FormatToPrettyDatePipe = new FormatToPrettyDatePipe();

  it('should create', () => {
    expect(pipe).toBeTruthy();
  });

  it('should return formatted for this year', () => {
    const year: number = new Date().getFullYear();
    const date: Date = new Date(year + '-01-01');

    const result: string = pipe.transform(date);

    expect(result).toBe('01. Jan.');
  });

  it('should format in "dd.MM.yyyy"', () => {
    const date: Date = new Date();
    date.setFullYear(2050, 4, 10);

    const result: string = pipe.transform(date);

    expect(result).toBe('10.05.2050');
  });
});
