/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { Action, createAction, createFeatureSelector, createSelector, props } from '@ngrx/store';
import {
  LoadResourceFailureProps,
  LoadResourceSuccessProps,
  ResourceUriProps,
  SingleResourceLoadActions,
  TypedActionCreator,
  TypedActionCreatorWithProps,
} from '../src/lib/ngrx/actions';
import { SingleResourceState } from '../src/lib/ngrx/state.model';
import { StateInfo } from '../src/lib/resource/resource.model';

export const DUMMY_ACTION: Action<string> = { type: 'Dummy Action' };

export const FUNCTIONAL_FEATURE_KEY = 'FunctionalState';
export const FUNCTIONAL_FEATURE_PATH_KEY = 'FunctionalResourcePath';

export interface SingleResourceParentState {
  readonly [FUNCTIONAL_FEATURE_KEY]: SingleResourcePartialState;
}
export interface SingleResourcePartialState {
  readonly [FUNCTIONAL_FEATURE_PATH_KEY]: SingleResourceState;
}

export class StateInfoTestFactory {
  public static readonly STATE_INFO_NAME: string = FUNCTIONAL_FEATURE_KEY;
  public static readonly STATE_INFO_PATH: string = FUNCTIONAL_FEATURE_PATH_KEY;

  public static create(): StateInfo {
    return {
      name: this.STATE_INFO_NAME,
      path: this.STATE_INFO_PATH,
    };
  }
}

export const dummySelector = createSelector(
  createFeatureSelector(FUNCTIONAL_FEATURE_KEY),
  (state: unknown) => state,
);

export const dummyLoadAction: TypedActionCreatorWithProps<ResourceUriProps> = createAction(
  'DummyLoadAction',
  props<ResourceUriProps>(),
);

export const dummyLoadSuccessAction: TypedActionCreatorWithProps<LoadResourceSuccessProps> =
  createAction('DummyLoadSuccessAction', props<LoadResourceSuccessProps>());

export const dummyLoadFailureAction: TypedActionCreatorWithProps<LoadResourceFailureProps> =
  createAction('DummyLoadFailureAction', props<LoadResourceFailureProps>());

export const dummyClearAction: TypedActionCreator = createAction('DummyClearAction');

export const dummyReloadAction: TypedActionCreator = createAction('DummyReloadAction');

export const singleResourceActions: SingleResourceLoadActions = {
  loadAction: dummyLoadAction,
  loadSuccessAction: dummyLoadSuccessAction,
  loadFailureAction: dummyLoadFailureAction,
  clearAction: dummyClearAction,
  reloadAction: dummyReloadAction,
};
