import { OzgCloudComponentFactory } from '@alfa-client/tech-shared';
import { dispatchEventFromFixture, Mock, mock, MockEvent, mockGetValue } from '@alfa-client/test-utils';
import { DIALOG_COMPONENT, OzgcloudDialogService } from '@alfa-client/ui-shared';
import { DIALOG_DATA } from '@angular/cdk/dialog';
import { ComponentType } from '@angular/cdk/portal';
import { ComponentRef, Injector, ViewContainerRef } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { expect } from '@jest/globals';
import { ButtonComponent } from '@ods/system';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { OpenDialogButtonComponent } from './open-dialog-button.component';

describe('OpenDialogButtonComponent', () => {
  let component: OpenDialogButtonComponent;
  let fixture: ComponentFixture<OpenDialogButtonComponent>;

  let dialogComponent: Mock<ComponentType<any>>;
  let componentFactory: Mock<OzgCloudComponentFactory>;
  let dialogService: Mock<OzgcloudDialogService>;
  let viewContainerRef: Mock<ViewContainerRef>;
  let injector: Mock<Injector>;

  const openDialog: string = getDataTestIdOf('open-dialog');

  const componentRef: ComponentRef<any> = <any>{ instance: { constructor: null } };
  const dialogResponse: any = {};
  const dummyDialogData: any = { someField: 'someValue' };

  beforeEach(async () => {
    dialogComponent = {};
    componentFactory = mock(OzgCloudComponentFactory);
    dialogService = { ...mock(OzgcloudDialogService), openInContext: jest.fn().mockReturnValue({ closed: of(dialogResponse) }) };
    viewContainerRef = mock(ViewContainerRef as any);
    injector = mock(Injector as any);

    await TestBed.configureTestingModule({
      imports: [OpenDialogButtonComponent],
      declarations: [MockComponent(ButtonComponent)],
      providers: [
        {
          provide: DIALOG_COMPONENT,
          useValue: dialogComponent,
        },
        {
          provide: OzgCloudComponentFactory,
          useValue: componentFactory,
        },
        {
          provide: OzgcloudDialogService,
          useValue: dialogService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(OpenDialogButtonComponent);
    component = fixture.componentInstance;
    mockGetValue(component, 'viewContainerRef', viewContainerRef);
    mockGetValue(component, 'injector', injector);
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('on button click', () => {
    beforeEach(() => {
      component._createComponent = jest.fn().mockReturnValue(componentRef);
    });

    it('should call create component factory to create component', () => {
      dispatchEventFromFixture(fixture, openDialog, MockEvent.CLICK);

      expect(component._createComponent).toHaveBeenCalled();
    });

    it('should call dialog service to open dialog', () => {
      component.dialogData = dummyDialogData;

      dispatchEventFromFixture(fixture, openDialog, MockEvent.CLICK);

      expect(dialogService.openInContext).toHaveBeenCalledWith(
        componentRef.instance.constructor,
        viewContainerRef,
        dummyDialogData,
      );
    });
  });

  describe('create component', () => {
    it('should call component factory to create component', () => {
      component.dialogData = dummyDialogData;

      component._createComponent();

      expect(componentFactory.createComponent).toHaveBeenCalledWith(dialogComponent, injector, [
        { provide: DIALOG_DATA, useValue: dummyDialogData },
      ]);
    });
  });
});
