/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { isClipboardReadSupported } from '@alfa-client/tech-shared';
import { existsAsHtmlElement, notExistsAsHtmlElement } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ButtonComponent, IconComponent, TooltipDirective } from '@ods/system';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent, MockDirective } from 'ng-mocks';
import { OzgcloudPasteTextButtonComponent } from './ozgcloud-paste-text-button.component';

jest.mock('@alfa-client/tech-shared');
const isClipboardReadSupportedMock = isClipboardReadSupported as jest.Mock;

const clipboardContent = '1234';
const readTextMock = jest.fn().mockResolvedValue(clipboardContent);
Object.assign(navigator, { clipboard: { readText: readTextMock } });
const pasteButtonLocator: string = getDataTestIdOf('paste-button');

describe('OzgcloudPasteTextButtonComponent', () => {
  let component: OzgcloudPasteTextButtonComponent;
  let fixture: ComponentFixture<OzgcloudPasteTextButtonComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      imports: [
        OzgcloudPasteTextButtonComponent,
        MockDirective(TooltipDirective),
        MockComponent(ButtonComponent),
        MockComponent(IconComponent),
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(OzgcloudPasteTextButtonComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('should have no paste supported by default', () => {
    expect(component.isPasteSupported).toBeFalsy();
  });

  it('should not render if paste not supported', () => {
    notExistsAsHtmlElement(fixture, pasteButtonLocator);
  });

  describe('ngOnInit', () => {
    it('should check if clipboard read is supported', () => {
      component.ngOnInit();

      expect(isClipboardReadSupported).toHaveBeenCalled();
    });

    it('should set paste support', () => {
      isClipboardReadSupportedMock.mockReturnValue(true);

      component.ngOnInit();

      expect(component.isPasteSupported).toBeTruthy();
    });

    it('should render if paste supported', () => {
      isClipboardReadSupportedMock.mockReturnValue(true);
      fixture.detectChanges();

      component.ngOnInit();

      existsAsHtmlElement(fixture, pasteButtonLocator);
    });
  });

  describe('onPaste', () => {
    it('should emit clipboard content', async () => {
      jest.spyOn(component.pasteEmitter, 'emit');

      await component.onPaste();

      expect(component.pasteEmitter.emit).toHaveBeenCalledWith(clipboardContent);
    });
  });
});
