/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BeschiedenDateInVorgangContainerComponent } from '@alfa-client/bescheid';
import { EMPTY_STRING, EnumToLabelPipe, HasLinkPipe } from '@alfa-client/tech-shared';
import { existsAsHtmlElement, getMockComponent, notExistsAsHtmlElement } from '@alfa-client/test-utils';
import { VorgangHeaderLinkRel } from '@alfa-client/vorgang-shared';
import { VorgangNummerComponent, VorgangStatusDotComponent, VorgangStatusTextComponent } from '@alfa-client/vorgang-shared-ui';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatIconTestingModule } from '@angular/material/icon/testing';
import { IconComponent } from '@ods/system';
import { MockComponent } from 'ng-mocks';
import { getDataTestIdOf } from '../../../../../../tech-shared/test/data-test';
import { UserProfileInVorgangContainerComponent } from '../../../../../../user-profile/src/lib/user-profile-in-vorgang-container/user-profile-in-vorgang-container.component';
import { createVorgangWithEingangResource } from '../../../../../../vorgang-shared/test/vorgang';
import { AktenzeichenEditableComponent } from '../../../aktenzeichen-editable/aktenzeichen-editable.component';
import { VorgangDetailHeaderComponent } from './vorgang-detail-header.component';

describe('VorgangDetailHeaderComponent', () => {
  let component: VorgangDetailHeaderComponent;
  let fixture: ComponentFixture<VorgangDetailHeaderComponent>;

  const user: string = getDataTestIdOf('vorgang-header-user-icon');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        VorgangDetailHeaderComponent,
        EnumToLabelPipe,
        HasLinkPipe,
        MockComponent(UserProfileInVorgangContainerComponent),
        MockComponent(VorgangStatusDotComponent),
        MockComponent(VorgangStatusTextComponent),
        MockComponent(AktenzeichenEditableComponent),
        MockComponent(VorgangNummerComponent),
        MockComponent(BeschiedenDateInVorgangContainerComponent),
        MockComponent(IconComponent),
      ],
      imports: [MatIconTestingModule],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(VorgangDetailHeaderComponent);
    component = fixture.componentInstance;
    component.vorgangWithEingang = createVorgangWithEingangResource();
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('component', () => {
    describe('set vorgangWithEingang', () => {
      it('should have Aktenzeichen', () => {
        expect(component.hasAktenzeichen).toBeTruthy();
      });

      it.each([null, EMPTY_STRING])('should NOT have aktenzeichen for %s', (aktenzeichen: string) => {
        const vorgangWithEingang = createVorgangWithEingangResource();
        vorgangWithEingang.aktenzeichen = aktenzeichen;
        component.vorgangWithEingang = vorgangWithEingang;

        expect(component.hasAktenzeichen).toBeFalsy();
      });
    });
  });

  describe('template', () => {
    describe('bearbeiter/user icon', () => {
      it('should be visible', () => {
        component.vorgangWithEingang = createVorgangWithEingangResource([VorgangHeaderLinkRel.ASSIGN]);
        fixture.detectChanges();

        existsAsHtmlElement(fixture, user);
      });

      it('should NOT be visible', () => {
        component.vorgangWithEingang = createVorgangWithEingangResource();
        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, user);
      });
    });

    describe('aktenzeichen icon', () => {
      it('should be text filled', () => {
        const icon: IconComponent = getMockComponent(fixture, IconComponent);

        expect(icon.fill).toBe('text');
      });

      it('should be neutral filled', () => {
        component.hasAktenzeichen = false;

        fixture.detectChanges();
        const icon: IconComponent = getMockComponent(fixture, IconComponent);

        expect(icon.fill).toBe('neutral');
      });
    });
  });
});
