import { CommandResource, isSuccessfulDone } from '@alfa-client/command-shared';
import { StateResource } from '@alfa-client/tech-shared';
import { VorgangService, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { DialogRef } from '@angular/cdk/dialog';
import { Component, inject } from '@angular/core';
import { Observable, tap } from 'rxjs';
import { VorgangArchiveConfirmationDialogData } from './vorgang-archive-confirmation-dialog/vorgang-archive-confirmation-dialog.model';

@Component({
  selector: 'alfa-vorgang-archive-confirmation-dialog-container',
  templateUrl: './vorgang-archive-confirmation-dialog-container.component.html',
})
export class VorgangArchiveConfirmationDialogContainerComponent {
  private vorgangService = inject(VorgangService);
  private dialogRef = inject(DialogRef<unknown, VorgangArchiveConfirmationDialogData>);

  private vorgangWithEingang: VorgangWithEingangResource = this.dialogRef.config.data.vorgangWithEingang;
  public archiveStateResource$: Observable<StateResource<CommandResource>>;

  archiveDoneHandler(): void {
    this.dialogRef.close();
  }

  public onCancelButtonClicked(): void {
    this.dialogRef.close();
  }

  public onConfirmationButtonClicked(): void {
    this.archiveStateResource$ = this.vorgangService.archive(this.vorgangWithEingang).pipe(
      tap((command: StateResource<CommandResource>) => {
        if (isSuccessfulDone(command.resource)) this.archiveDoneHandler();
      }),
    );
  }
}
