/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { HasLinkPipe, createStateResource } from '@alfa-client/tech-shared';
import { WiedervorlageListLinkRel } from '@alfa-client/wiedervorlage-shared';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { RouterTestingModule } from '@angular/router/testing';
import { ButtonWithSpinnerComponent } from '@ods/component';
import { IconComponent } from '@ods/system';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { createWiedervorlageListResource } from 'libs/wiedervorlage-shared/test/wiedervorlage';
import { MockComponent } from 'ng-mocks';
import { WiedervorlageCreateButtonComponent } from './wiedervorlage-create-button.component';

describe('WiedervorlageCreateButtonComponent', () => {
  let component: WiedervorlageCreateButtonComponent;
  let fixture: ComponentFixture<WiedervorlageCreateButtonComponent>;

  const wiedervorlageButton: string = getDataTestIdOf('create-wiedervorlage');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        WiedervorlageCreateButtonComponent,
        HasLinkPipe,
        MockComponent(ButtonWithSpinnerComponent),
        MockComponent(IconComponent),
      ],
      imports: [RouterTestingModule],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(WiedervorlageCreateButtonComponent);
    component = fixture.componentInstance;
    component.wiedervorlageListStateResource = createStateResource(createWiedervorlageListResource());
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('create Wiedervorlage button', () => {
    it('should be visible', () => {
      component.wiedervorlageListStateResource = createStateResource(
        createWiedervorlageListResource([WiedervorlageListLinkRel.CREATE_WIEDERVORLAGE]),
      );

      fixture.detectChanges();
      const button = fixture.nativeElement.querySelector(wiedervorlageButton);

      expect(button).toBeInstanceOf(HTMLElement);
    });

    it('should be hidden', () => {
      component.wiedervorlageListStateResource = createStateResource(createWiedervorlageListResource());

      fixture.detectChanges();
      const button = fixture.nativeElement.querySelector(wiedervorlageButton);

      expect(button).not.toBeInstanceOf(HTMLElement);
    });
  });
});
