/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BinaryFileModule } from '@alfa-client/binary-file';
import {
  ConvertForDataTestPipe,
  FormatToPrettyDatePipe,
  HasLinkPipe,
  ToEmbeddedResourcesPipe,
  ToResourceUriPipe,
  ToTrafficLightPipe,
  ToTrafficLightTooltipPipe,
} from '@alfa-client/tech-shared';
import {
  BackButtonComponent,
  DateEditorComponent,
  ExpansionPanelComponent,
  IconButtonWithSpinnerComponent,
  OzgcloudStrokedButtonWithSpinnerComponent,
  OzgcloudTextEditorComponent,
  SpinnerComponent,
  SubnavigationComponent,
  TextAreaEditorComponent,
} from '@alfa-client/ui';
import { UserProfileModule } from '@alfa-client/user-profile';
import { VorgangSharedModule } from '@alfa-client/vorgang-shared';
import { VorgangSharedUiModule } from '@alfa-client/vorgang-shared-ui';
import { CommonModule, DatePipe } from '@angular/common';
import { NgModule } from '@angular/core';
import { ReactiveFormsModule } from '@angular/forms';
import { MatIcon } from '@angular/material/icon';
import { MatTooltip } from '@angular/material/tooltip';
import { RouterModule, Routes } from '@angular/router';
import { ButtonWithSpinnerComponent } from '@ods/component';
import { IconComponent, TooltipDirective } from '@ods/system';
import { MultiFileUploadComponent } from '../../../binary-file/src/lib/multi-file-upload/multi-file-upload.component';
import { CreateWiedervorlageButtonContainerComponent } from './create-wiedervorlage-button-container/create-wiedervorlage-button-container.component';
import { ErledigenButtonContainerComponent } from './erledigen-button-container/erledigen-button-container.component';
import { SubmitWiedervorlageButtonComponent } from './submit-wiedervorlage-button/submit-wiedervorlage-button.component';
import { WiedereroeffnenButtonContainerComponent } from './wiedereroeffnen-button-container/wiedereroeffnen-button-container.component';
import { WiedervorlageListInVorgangContainerComponent } from './wiedervorlage-list-in-vorgang-container/wiedervorlage-list-in-vorgang-container.component';
import { WiedervorlageCreateButtonComponent } from './wiedervorlage-list-in-vorgang-container/wiedervorlage-list-in-vorgang/wiedervorlage-create-button/wiedervorlage-create-button.component';
import { WiedervorlageInVorgangExpandButtonComponent } from './wiedervorlage-list-in-vorgang-container/wiedervorlage-list-in-vorgang/wiedervorlage-in-vorgang/wiedervorlage-in-vorgang-expand-button/wiedervorlage-in-vorgang-expand-button.component';
import { WiedervorlageInVorgangComponent } from './wiedervorlage-list-in-vorgang-container/wiedervorlage-list-in-vorgang/wiedervorlage-in-vorgang/wiedervorlage-in-vorgang.component';
import { WiedervorlageListInVorgangComponent } from './wiedervorlage-list-in-vorgang-container/wiedervorlage-list-in-vorgang/wiedervorlage-list-in-vorgang.component';
import { WiedervorlageListInVorgangListContainerComponent } from './wiedervorlage-list-in-vorgang-list-container/wiedervorlage-list-in-vorgang-list-container.component';
import { WiedervorlagePageContainerComponent } from './wiedervorlage-page-container/wiedervorlage-page-container.component';
import { WiedervorlageActionButtonsComponent } from './wiedervorlage-page-container/wiedervorlage-page/wiedervorlage-action-buttons/wiedervorlage-action-buttons.component';
import { WiedervorlageBreadcrumbContainerComponent } from './wiedervorlage-page-container/wiedervorlage-page/wiedervorlage-breadcrumb-container/wiedervorlage-breadcrumb-container.component';
import { WiedervorlageBreadcrumbComponent } from './wiedervorlage-page-container/wiedervorlage-page/wiedervorlage-breadcrumb-container/wiedervorlage-breadcrumb/wiedervorlage-breadcrumb.component';
import { WiedervorlageFormComponent } from './wiedervorlage-page-container/wiedervorlage-page/wiedervorlage-form/wiedervorlage-form.component';
import { WiedervorlagePageComponent } from './wiedervorlage-page-container/wiedervorlage-page/wiedervorlage-page.component';
import { WiedervorlageStatusComponent } from './wiedervorlage-status/wiedervorlage-status.component';
import { wiedervorlageGuard } from './wiedervorlage.guard';

const routes: Routes = [
  {
    path: 'wiedervorlage/:wiedervorlageUrl',
    component: WiedervorlagePageContainerComponent,
    title: 'Wiedervorlage zum Vorgang | Alfa',
    canActivate: [wiedervorlageGuard],
  },
];

@NgModule({
  imports: [
    CommonModule,
    RouterModule.forChild(routes),
    VorgangSharedModule,
    VorgangSharedUiModule,
    BinaryFileModule,
    UserProfileModule,
    HasLinkPipe,
    IconButtonWithSpinnerComponent,
    OzgcloudStrokedButtonWithSpinnerComponent,
    MatIcon,
    MatTooltip,
    ToTrafficLightTooltipPipe,
    ToResourceUriPipe,
    FormatToPrettyDatePipe,
    SpinnerComponent,
    ExpansionPanelComponent,
    ToEmbeddedResourcesPipe,
    BackButtonComponent,
    ReactiveFormsModule,
    OzgcloudTextEditorComponent,
    TextAreaEditorComponent,
    DateEditorComponent,
    SubnavigationComponent,
    ToTrafficLightPipe,
    ConvertForDataTestPipe,
    ButtonWithSpinnerComponent,
    TooltipDirective,
    IconComponent,
    MultiFileUploadComponent,
  ],
  declarations: [
    WiedervorlagePageComponent,
    WiedervorlageInVorgangComponent,
    WiedervorlageFormComponent,
    WiedervorlageBreadcrumbComponent,
    WiedervorlageListInVorgangContainerComponent,
    WiedervorlageListInVorgangComponent,
    WiedervorlageActionButtonsComponent,
    WiedervorlagePageContainerComponent,
    WiedervorlageStatusComponent,
    WiedervorlageCreateButtonComponent,
    WiedervorlageListInVorgangListContainerComponent,
    SubmitWiedervorlageButtonComponent,
    WiedereroeffnenButtonContainerComponent,
    ErledigenButtonContainerComponent,
    CreateWiedervorlageButtonContainerComponent,
    WiedervorlageBreadcrumbContainerComponent,
    WiedervorlageInVorgangExpandButtonComponent,
  ],
  providers: [DatePipe],
  exports: [
    WiedervorlageListInVorgangContainerComponent,
    WiedervorlageListInVorgangListContainerComponent,
    CreateWiedervorlageButtonContainerComponent,
  ],
})
export class WiedervorlageModule {}
