import { ZUSTAENDIGE_STELLE_SERVICE, ZustaendigeStelleService } from '@alfa-client/zustaendige-stelle-shared';
import { Component, EventEmitter, HostListener, Inject, Input, OnInit, Output } from '@angular/core';
import { Resource } from '@ngxp/rest';
import { InstantSearchResult } from '@ods/system';
import { Observable } from 'rxjs';

@Component({
  selector: 'alfa-search-zustaendige-stelle-form-container',
  templateUrl: './search-zustaendige-stelle-form-container.component.html',
})
export class SearchZustaendigeStelleFormContainerComponent<T extends Resource> implements OnInit {
  @Output() searchResultSelected = new EventEmitter<T>();
  @Input() focusOnSearchField: boolean = false;

  public searchResults$: Observable<InstantSearchResult<T>[]>;

  constructor(
    @Inject(ZUSTAENDIGE_STELLE_SERVICE)
    private service: ZustaendigeStelleService<T>,
  ) {}

  ngOnInit(): void {
    this.searchResults$ = this.service.getSearchResultList();
  }

  @HostListener('document:keydown', ['$event'])
  onKeyDownHandler(e: KeyboardEvent) {
    if (e.key === 'Enter' && this.isNotButtonElement(e.target as Element)) {
      e.preventDefault();
    }
  }

  public search(searchBy: string): void {
    this.service.search(searchBy);
  }

  public selectSearchResult(zustaendigeStelle: T): void {
    this.service.selectSearchResult(zustaendigeStelle);
    this.service.clearSearchResult();
    this.searchResultSelected.emit(zustaendigeStelle);
  }

  public clearSearchResult(): void {
    this.service.clearSearchResult();
  }

  public isNotButtonElement(e: Element): boolean {
    return e.tagName !== 'BUTTON';
  }
}
